#include "addmenuitemdialog.h"
#include "main.h"
#include "resource.h"
#include "config.h"
#include "langcode.h"

AddMenuItemDialog::AddMenuItemDialog(MenuDialog * menuDialog, MenuPtr menu, int groupIndex,
                                     int x, int y, int distance)
{
	m_MenuDialog = menuDialog;
	m_Menu = menu;
	m_GroupIndex = groupIndex;
	m_X = x;
	m_Y = y;
	m_Distance = distance;
}
AddMenuItemDialog::~AddMenuItemDialog()
{
}

int AddMenuItemDialog::DoModal()
{
	switch (m_Menu->GetType()) 
	{
	case MENU_ROOT:
		for (int index = 0; index < m_Menu->GetGroup()->GetTitleCount(); index++) {
			m_MenuItems.push_back(MenuItemPtr(new MenuItem(m_Menu, ROOTMENU_PLAYTITLE, index)));
		}
		switch (m_Menu->GetGroup()->GetMenuType()) {
		case MENUTYPE_COMBINED:
			m_MenuItems.push_back(MenuItemPtr(new MenuItem(m_Menu, ROOTMENU_GOTOCOMBINED, 0)));
			break;
		case MENUTYPE_SEPARATE:
			if (m_Menu->GetGroup()->HasAudioMenu()) {
				m_MenuItems.push_back(MenuItemPtr(new MenuItem(m_Menu, ROOTMENU_GOTOAUDIO, 0)));
			}
			if (m_Menu->GetGroup()->HasAudioMenu()) {
				m_MenuItems.push_back(MenuItemPtr(new MenuItem(m_Menu, ROOTMENU_GOTOSUBTITLE, 0)));
			}
			break;
		}
		for (int index = 0; index < m_Menu->GetGroup()->GetProject()->GetGroupCount(); index++) {
			if (index != m_GroupIndex) {
				m_MenuItems.push_back(MenuItemPtr(new MenuItem(m_Menu, ROOTMENU_GOTOTITLESET, index)));
			}
		}
		m_MenuItems.push_back(MenuItemPtr(new MenuItem(m_Menu, ROOTMENU_RESUME, 0)));
		break;

	case MENU_COMBINED:
		for (int aindex = 0; aindex < m_Menu->GetGroup()->GetAudioCount(); aindex++) {
			for (int sindex = 0; sindex <= m_Menu->GetGroup()->GetSubtitleCount(); sindex++) {
				m_MenuItems.push_back(MenuItemPtr(new MenuItem(m_Menu, aindex, sindex)));
			}
		}
		break;

	case MENU_AUDIO:
		for (int aindex = 0; aindex < m_Menu->GetGroup()->GetAudioCount(); aindex++) {
			m_MenuItems.push_back(MenuItemPtr(new MenuItem(m_Menu, aindex, 0)));
		}
		break;

	case MENU_SUBTITLE:
		for (int sindex = 0; sindex <= m_Menu->GetGroup()->GetSubtitleCount(); sindex++) {
			m_MenuItems.push_back(MenuItemPtr(new MenuItem(m_Menu, 0, sindex)));
		}
		break;
	}

	if (Dialog::DoModal(hInstance, MAKEINTRESOURCE(IDD_ADDMENUITEM), m_MenuDialog->hWnd())) {
		m_Menu->ClearSelection();
		int width  = m_Menu->GetGroup()->GetProject()->GetVideoWidth();
		int height = m_Menu->GetGroup()->GetProject()->GetVideoHeight();
		int x = m_X % width;
		int y = m_Y % height;
		for (MenuItemVector::iterator mii = m_MenuItems.begin(); mii != m_MenuItems.end(); mii++) {
			String caption = (*mii)->GetShortDescr();
			if (m_Menu->GetType() == MENU_ROOT) {
				if (g_Config->GetPlayTextFromFileName()) {
					if ((*mii)->GetSelector1() == ROOTMENU_PLAYTITLE) {
						caption = m_Menu->GetGroup()->GetTitle((*mii)->GetSelector2())->GetVideo(0)->GetFileName().toFileName();
					}
				}
			} else if (g_Config->GetSelectTextFromLanguage()) {
				switch (m_Menu->GetType()) {
				case MENU_COMBINED:
					caption = LanguageName[m_Menu->GetGroup()->GetAudioLanguage((*mii)->GetSelector1())];
					if ((*mii)->GetSelector2()) {
						caption += S(", ") + LanguageName[m_Menu->GetGroup()->GetSubtitleLanguage((*mii)->GetSelector2() - 1)] + S(" subtitles");
					}
					break;
				case MENU_AUDIO:
					caption = LanguageName[m_Menu->GetGroup()->GetAudioLanguage((*mii)->GetSelector1())];
					break;
				case MENU_SUBTITLE:
					if ((*mii)->GetSelector2()) {
						caption = LanguageName[m_Menu->GetGroup()->GetSubtitleLanguage((*mii)->GetSelector2() - 1)];
					}
					break;
				}
			}
			(*mii)->SetCaption(caption, false);
			(*mii)->SetX(x, false);
			(*mii)->SetY(y, false);
			m_Menu->AddMenuItem(*mii);
			y = (y + m_Distance) % height;
		}
		return 1;
	}
	return 0;
}

bool AddMenuItemDialog::DialogProc(UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
	case WM_INITDIALOG:
		for (MenuItemVector::iterator mii = m_MenuItems.begin(); mii != m_MenuItems.end(); mii++) {
			SendDlgItemMessage(hWnd(), IDC_ITEMLIST, LB_ADDSTRING, 0, (LPARAM) (*mii)->GetShortDescr().c_str());
		}
		SetDlgItemText(hWnd(), IDC_X, String(m_X).c_str());
		SetDlgItemText(hWnd(), IDC_Y, String(m_Y).c_str());
		SetDlgItemText(hWnd(), IDC_DISTANCE, String(m_Distance).c_str());
		return true;
	case WM_COMMAND:
		switch (LOWORD(wParam)) 
		{
		case IDOK:
			if (HIWORD(wParam) == BN_CLICKED) {
				int count = 0;
				int index = 0;
				MenuItemVector::iterator mii = m_MenuItems.begin();
				while (mii != m_MenuItems.end()) {
					if (SendDlgItemMessage(hWnd(), IDC_ITEMLIST, LB_GETSEL, index, 0)) {
						mii++;
						count++;
					} else {
						mii = m_MenuItems.erase(mii);
					}
					index++;
				}
				if (count) {
					m_X = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_X)).toAnsi().c_str());
					m_Y = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_Y)).toAnsi().c_str());
					m_Distance = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_DISTANCE)).toAnsi().c_str());
					EndDialog(hWnd(), m_MenuItems.size());
				}
				return true;
			}
			return false;
		case IDCANCEL:
			if (HIWORD(wParam) == BN_CLICKED) {
				EndDialog(hWnd(), 0);
				return true;
			}
			return false;
		}
		return false;
	case WM_CLOSE:
		EndDialog(hWnd(), 0);
		return true;
	}
	return false;
}


