#ifndef AUDIO_H_
#define AUDIO_H_

#include "managableptr.h"
#include "managedptr.h"
#include "string.h"
#include "csvread/csvread.h"
#include "avswrapper.h"

#include <vector>

class Video;

#define AUDIOBITRATE_MAX 6

class Audio : public ManagablePtr
{
public:
	Audio(Video * title, Audio * audio = NULL);
	virtual ~Audio();
	Video * GetVideo() const;
	bool IsFileSelected() const;
	void SetFileSelected(bool selected);
	bool IsTrackSelected() const;
	void SetTrackSelected(bool selected);

public:
	bool GetFromVideo() const;
	bool SetFromVideo(bool fromVideo, bool setChanged = true);
	const String & GetFileName() const;
	bool SetFileName(const String & fileName, bool setChanged = true);
	int GetBitRate() const;
	bool SetBitRate(int bitRate, bool setChanged = true);
	String GetOpener() const;
	bool SetOpener(const String & opener, bool setChanged = true);
	const String & GetPreProcessor() const;
	bool SetPreProcessor(const String & preProcessor, bool setChanged = true);
	
	void SetInvalid(bool clearChecked);
	bool Analyze();
	bool IsChecked() const;
	bool IsValid() const;
	bool IsScript() const;
	bool IsAVI() const;
	const AVSWrapper::Info & GetInfo() const;
	int GetNoOfChannels() const;
	int GetBitRateInKbps() const;
	int GetIndex() const;
	void SetIndex(int index);
	String GetIdent(bool forTrack);
	String Recalculate();
	void SetChanged();

	void GetDescription(StringVectorVector & strings, bool forTitle) const;
	bool Load(csv_file_t csv, int & lineno, int nest);
	void Save(FILE * f, int nest) const;

protected:
	Video * m_Video;
	bool    m_FromVideo;
	String  m_FileName;
	int     m_BitRate;
	String  m_Opener;
	String  m_PreProcessor;
	bool    m_IsChecked;
	AVSWrapper::Info m_Info;
	String  m_OpenError;
	int     m_Index;
	bool    m_FileSelected;
	bool    m_TrackSelected;
};

typedef ManagedPtr<Audio> AudioPtr;
typedef std::vector<AudioPtr> AudioVector;

#endif // AUDIO_H_
