#include "audioencoder.h"
#include "group.h"
#include "title.h"
#include "video.h"
#include "audio.h"
#include "tools.h"
#include "main.h"
#include "config.h"

#include "winutils/winutils.h"

#include <stdio.h>
#include <map>

AudioEncoder::AudioEncoder(ProjectPtr project, int groupIndex, int titleIndex, int audioIndex) : Thread()
{
	m_Project = project;
	m_GroupIndex = groupIndex;
	m_TitleIndex = titleIndex;
	m_AudioIndex = audioIndex;
}

AudioEncoder::~AudioEncoder()
{
}

String AudioEncoder::GetDescription() const
{
	return S("Encoding audio track ") + String(m_AudioIndex + 1) 
		+ S(" of title ") + String(m_TitleIndex + 1) + S(" in group ") + String(m_GroupIndex + 1);
}

bool AudioEncoder::OnRun()
{
	bool ok = true;
	
	GroupPtr group = m_Project->GetGroup(m_GroupIndex);
	TitlePtr title = group->GetTitle(m_TitleIndex);

	String baseFileName = S("set") + String(m_GroupIndex + 1) + 
		S("title") + String(m_TitleIndex + 1) + 
		S("audio") + String(m_AudioIndex + 1);
	String avsFileName = String(m_Project->GetOutputDirectory().toShortPath() + S("\\") + baseFileName + S(".avs"));
	// String wavFileName = String(m_Project->GetOutputDirectory().toShortPath() + S("\\") + baseFileName + S(".wav"));
	String ac3FileName = String(m_Project->GetOutputDirectory().toShortPath() + S("\\") + baseFileName + S(".ac3"));
	String logFileName = String(m_Project->GetOutputDirectory() + S("\\") + S("log.txt")).toFullPath();

	AddFileToDelete(avsFileName);
	// AddFileToDelete(wavFileName);

	FILE * f = fopen(avsFileName.toAnsi().c_str(), "wt");
	if (f) {
		std::map<String, bool> imports;
		for (int vindex = 0; vindex < title->GetVideoCount(); vindex++) {
			VideoPtr video = title->GetVideo(vindex);
			AudioPtr audio = video->GetAudio(m_AudioIndex);
			if (video->GetOpener().length()) {
				Config::FunctionMap::const_iterator opener = g_Config->GetOpeners().find(video->GetOpener());
				if (opener != g_Config->GetOpeners().end()) imports[(*opener).second] = true;
			}
			if (video->GetPreProcessor().length()) {
				Config::FunctionMap::const_iterator preProcessor = g_Config->GetPreProcessors().find(video->GetPreProcessor());
				if (preProcessor != g_Config->GetPreProcessors().end()) imports[(*preProcessor).second] = true;
			}
			if (audio->GetOpener().length() && ! audio->GetFromVideo()) {
				Config::FunctionMap::const_iterator opener = g_Config->GetOpeners().find(audio->GetOpener());
				if (opener != g_Config->GetOpeners().end()) imports[(*opener).second] = true;
			}
			if (audio->GetPreProcessor().length()) {
				Config::FunctionMap::const_iterator preProcessor = g_Config->GetPreProcessors().find(audio->GetPreProcessor());
				if (preProcessor != g_Config->GetPreProcessors().end()) imports[(*preProcessor).second] = true;
			}
		}
		for (std::map<String, bool>::iterator import = imports.begin(); import != imports.end(); import++) {
			fprintf(f, "Import(%s)\n", (*import).first.enclose().toAnsi().c_str());
		}
		for (int vindex = 0; vindex < title->GetVideoCount(); vindex++) {
			VideoPtr video = title->GetVideo(vindex);
			AudioPtr audio = video->GetAudio(m_AudioIndex);
			if (video->IsScript()) {
				fprintf(f, "a%d = Import(%s)\n", vindex, video->GetFileName().enclose().toAnsi().c_str());
			} else {
				fprintf(f, "a%d = %s(%s, %s)\n", vindex, video->GetOpener().toAnsi().c_str(),
					video->GetFileName().enclose().toAnsi().c_str(),
					(m_Project->GetDVDFormat() == DVDFORMAT_PAL ? "true" : "false"));
				if (video->GetPreProcessor().length()) {
					fprintf(f, "a%d = %s(a%d, %s)\n", vindex, video->GetPreProcessor().toAnsi().c_str(), vindex,
						(m_Project->GetDVDFormat() == DVDFORMAT_PAL ? "true" : "false"));
				}
			}
			if (! audio->GetFromVideo()) {
				if (audio->IsScript()) {
					fprintf(f, "at = Import(%s)\n", audio->GetFileName().enclose().toAnsi().c_str());
				} else {
					fprintf(f, "at = %s(%s, %s)\n", audio->GetOpener().toAnsi().c_str(),
						audio->GetFileName().enclose().toAnsi().c_str(),
						(m_Project->GetDVDFormat() == DVDFORMAT_PAL ? "true" : "false"));
				}
				fprintf(f, "fc = a%d.FrameCount - 1\n", vindex);
				fprintf(f, "a%d = AudioDub(a%d, at)\n", vindex, vindex);
				fprintf(f, "a%d = Trim(a%d, 0, fc)\n", vindex, vindex);
			}
			if (audio->GetPreProcessor().length()) {
				fprintf(f, "a%d = %s(a%d, %s)\n", vindex, 
					audio->GetPreProcessor().toAnsi().c_str(), vindex,
					(m_Project->GetDVDFormat() == DVDFORMAT_PAL ? "true" : "false"));
			}
			fprintf(f, "a%d = ConvertAudioToFloat(a%d)\n", vindex, vindex);
			if (video->AssumeFPS()) {
				if (video->DoConvertFPS()) {
					fprintf(f, "a%d = ChangeFPS(a%d, %f)\n", vindex, vindex, video->ConvertToFPS());
				}
				fprintf(f, "a%d = AssumeFPS(a%d, \"%s\", true)\n", vindex, vindex,
					(m_Project->GetDVDFormat() == DVDFORMAT_PAL ? "pal_video" : "ntsc_video"));
			}
			fprintf(f, "try {\n");
			fprintf(f, "  a%d = SSRC(a%d, 48000)\n", vindex, vindex);
			fprintf(f, "} catch(err_msg) {\n");
			fprintf(f, "  a%d = ResampleAudio(a%d, 48000)\n", vindex, vindex);
			fprintf(f, "}\n");
			if (title->GetConvertToMono(m_AudioIndex)) {
				fprintf(f, "a%d = ConvertToMono(a%d)\n", vindex, vindex);
			}
			fprintf(f, "a%d = ConvertAudioTo16bit(a%d)\n", vindex, vindex);
		}
		if (title->GetVideoCount() == 1) {
			fprintf(f, "return KillVideo(a0)\n");
		} else {
			fprintf(f, "return KillVideo(AlignedSplice(a0");
			for (int vindex = 1; vindex < title->GetVideoCount(); vindex++) {
				fprintf(f, ", a%d", vindex);
			}
			fprintf(f, "))\n");
		}
		fclose(f);
	}

	TCHAR comspec[MAX_PATH];
	GetEnvironmentVariable(TEXT("COMSPEC"), comspec, MAX_PATH);
	String command = tools.wavi.toShortPath() + S(" ") + avsFileName + S(" - | ")
		+ tools.aften.toShortPath() + S(" - ") + ac3FileName + S(" -readtoeof 1 -v 0 -b ") 
		+ String(title->GetVideo(0)->GetAudio(m_AudioIndex)->GetBitRateInKbps());
	String params = g_Config->GetAftenParams().trim();
	if (params.length()) command += S(" ") + params;
	ok = ok && ExecuteCommand(comspec, (S("/C ") + command.enclose()).c_str(), 
		NULL, NULL, logFileName.c_str(), isCancelled, waitingForUser);
		
	if (ok) DeleteFilesNow();
	return ok;
}
