#ifndef CONFIG_H_
#define CONFIG_H_

#include "string.h"
#include <map>

#define ENCODER_HC  0
#define ENCODER_QU  1
#define ENCODER_MAX 2

#define HCPROFILE_FAST   0
#define HCPROFILE_NORMAL 1
#define HCPROFILE_BEST   2
#define HCPROFILE_MAX    3

class Config
{
public:
	Config();
	virtual ~Config();
	void Refresh();

public:
	bool SetDVDFormat(int dvdFormat, bool setChanged = true);
	int GetDVDFormat() const;
	bool SetDVDSize(int dvdSize, bool setChanged = true);
	int GetDVDSize() const;
	bool SetCustomSize(int customSize, bool setChanged = true);
	int GetCustomSize() const;
	bool SetCustomBitRate(int customBitRate, bool setChanged = true);
	int GetCustomBitRate() const;
	bool SetOutputType(int outputType, bool setChanged = true);
	int GetOutputType() const;
	COLORREF GetMenuColor(int which) const;
	bool SetMenuColor(int which, COLORREF color, bool setChanged = true);
	bool SetPlayTextFromFileName(bool playTextFromFileName, bool setChanged = true);
	bool GetPlayTextFromFileName() const;
	bool SetSelectTextFromLanguage(bool selectTextFromLanguage, bool setChanged = true);
	bool GetSelectTextFromLanguage() const;
	bool GetNormalText() const;
	bool SetNormalText(bool normalText, bool setChanged = true);
	bool GetHiliteText() const;
	bool SetHiliteText(bool hiliteText, bool setChanged = true);
	const String & GetMenuFontFace();
	bool SetMenuFontFace(const String & fontFace, bool setChanged = true);
	int GetMenuFontSize() const;
	bool SetMenuFontSize(int fontSize, bool setChanged = true);
	bool GetMenuBold() const;
	bool SetMenuBold(bool bold, bool setChanged = true);
	bool GetMenuItalic() const;
	bool SetMenuItalic(bool italic, bool setChanged = true);
	bool GetNormalRect() const;
	bool SetNormalRect(bool normalRect, bool setChanged = true);
	bool GetHiliteRect() const;
	bool SetHiliteRect(bool hiliteRect, bool setChanged = true);
	int GetThickness() const;
	bool SetThickness(int thickness, bool setChanged = true);
	int GetHRadius() const;
	bool SetHRadius(int hRadius, bool setChanged = true);
	int GetVRadius() const;
	bool SetVRadius(int hRadius, bool setChanged = true);
	bool GetFillNormal() const;
	bool SetFillNormal(bool fillNormal, bool setChanged = true);
	bool GetFillHilite() const;
	bool SetFillHilite(bool fillHilite, bool setChanged = true);
	bool GetAutoResize() const;
	bool SetAutoResize(bool autoResize, bool setChanged = true);
	int GetHMargin() const;
	bool SetHMargin(int hMargin, bool setChanged = true);
	int GetVMargin() const;
	bool SetVMargin(int hMargin, bool setChanged = true);
	int GetAlignment() const;
	bool SetAlignment(int alignment, bool setChanged = true);
	bool GetCompensateOverscan() const;
	bool SetCompensateOverscan(bool compensateOverscan, bool setChanged = true);
	int GetOverscanPercent() const;
	bool SetOverscanPercent(int overscanPercent, bool setChanged = true);
	bool GetCreateChapters() const;
	bool SetCreateChapters(bool createChapters, bool setChanged = true);
	int  GetChapterCount() const;
	bool SetChapterCount(int chapterCount, bool setChanged = true);
	const String & GetFontFace() const;
	bool SetFontFace(const String & fontFace, bool setChanged = true);
	int GetFontSize() const;
	bool SetFontSize(int fontSize, bool setChanged = true);
	bool GetBold() const;
	bool SetBold(bool bold, bool setChanged = true);
	bool GetItalic() const;
	bool SetItalic(bool italic, bool setChanged = true);
	bool GetBox() const;
	bool SetBox(bool box, bool setChanged = true);
	COLORREF GetColor() const;
	bool SetColor(COLORREF color, bool setChanged = true);
	int GetEncoder() const;
	bool SetEncoder(int encoder, bool setChanged = true);
	bool GetHideEncoder() const;
	bool SetHideEncoder(bool hideEncoder, bool setChanged = true);
	int GetHCProfile() const;
	bool SetHCProfile(int hcProfile, bool setChanged = true);
	const String & GetHCOptions() const;
	bool SetHCOptions(const String & hcOptions, bool setChanged = true);
	bool GetQu2Pass() const;
	bool SetQu2Pass(bool qu2Pass, bool setChanged = true);
	bool GetQuHighQuality() const;
	bool SetQuHighQuality(bool quHighQuality, bool setChanged = true);
	bool GetQuTrellis() const;
	bool SetQuTrellis(bool quTrellis, bool setChanged = true);
	bool GetQuExtreme() const;
	bool SetQuExtreme(bool quExtreme, bool setChanged = true);
	const String & GetQuParams() const;
	bool SetQuParams(const String & quParams, bool setChanged = true);
	const String & GetAftenParams() const;
	bool SetAftenParams(const String & aftenParams, bool setChanged = true);
	const String & GetVideoOpener() const;
	bool SetVideoOpener(const String & opener, bool setChanged = true);
	const String & GetVideoPreProcessor() const;
	bool SetVideoPreProcessor(const String & preProcessor, bool setChanged = true);
	const String & GetAudioOpener() const;
	bool SetAudioOpener(const String & opener, bool setChanged = true);
	const String & GetAudioPreProcessor() const;
	bool SetAudioPreProcessor(const String & preProcessor, bool setChanged = true);
	const String & GetImgBurn() const;
	bool SetImgBurn(const String & imgBurn, bool setChanged = true);
	bool IsImgBurnValid() const;
	bool HasChanged() const;
	void SetChanged();
	void ClearChanged();
	bool Load(const String & fileName);
	bool Save(String fileName = S(""));

protected:
	int m_DVDFormat;
	int m_DVDSize;
	int m_CustomSize;
	int m_CustomBitRate;
	int m_OutputType;
	bool m_PlayTextFromFileName;
	bool m_SelectTextFromLanguage;
	bool m_NormalText;
	bool m_HiliteText;
	COLORREF m_MenuColor[4];
	String m_MenuFontFace;
	int m_MenuFontSize;
	bool m_MenuBold;
	bool m_MenuItalic;
	bool m_NormalRect;
	bool m_HiliteRect;
	int  m_Thickness;
	int  m_HRadius;
	int  m_VRadius;
	bool m_FillNormal;
	bool m_FillHilite;
	bool m_AutoResize;
	int  m_HMargin;
	int  m_VMargin;
	int  m_Alignment;
	bool m_CompensateOverscan;
	int  m_OverscanPercent;
	bool m_CreateChapters;
	int  m_ChapterCount;
	String   m_FontFace;
	int      m_FontSize;
	bool     m_Bold;
	bool     m_Italic;
	bool     m_Box;
	COLORREF m_Color;
	int  m_Encoder;
	bool m_HideEncoder;
	int  m_HCProfile;
	String m_HCOptions;
	bool m_Qu2Pass;
	bool m_QuHighQuality;
	bool m_QuTrellis;
	bool m_QuExtreme;
	String m_QuParams;
	String m_AftenParams;
	String m_VideoOpener;
	String m_VideoPreProcessor;
	String m_AudioOpener;
	String m_AudioPreProcessor;
	String m_ImgBurn;
	bool m_HasChanged;
	String m_ConfigFile;
	String m_FunctionDir;
	String m_FunctionMask;
	unsigned m_HashValue;

public:
	const String & GetFunctionDir();
	typedef std::map<String, String> FunctionMap;
	bool RefreshFunctionMaps();
	const FunctionMap & GetOpeners();
	const FunctionMap & GetPreProcessors();

protected:
	FunctionMap m_Openers;
	FunctionMap m_PreProcessors;
};

extern Config * g_Config;

#endif // CONFIG_H_