#include "configdialog.h"
#include "main.h"
#include "config.h"
#include "project.h"
#include "menu.h"
#include "menuitem.h"
#include "resource.h"
#include "tools.h"

ConfigDialog::ConfigDialog(Dialog * parent) : Dialog()
{
	m_Parent = parent;
}

ConfigDialog::~ConfigDialog()
{
}

int ConfigDialog::DoModal()
{
	return Dialog::DoModal(hInstance, MAKEINTRESOURCE(IDD_CONFIG), m_Parent->hWnd());
}

void ConfigDialog::Update()
{
	Config * config = g_Config; // don't ask me why...
	SendDlgItemMessage(hWnd(), IDC_DVDFORMAT_PAL, BM_SETCHECK,
		(config->GetDVDFormat() == DVDFORMAT_PAL ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_DVDFORMAT_NTSC, BM_SETCHECK,
		(config->GetDVDFormat() == DVDFORMAT_NTSC ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_DVDSIZE_DVD_5, BM_SETCHECK, 
		(config->GetDVDSize() == DVDSIZE_DVD_5 ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_DVDSIZE_DVD_9, BM_SETCHECK, 
		(config->GetDVDSize() == DVDSIZE_DVD_9 ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_DVDSIZE_CUSTOM, BM_SETCHECK, 
		(config->GetDVDSize() == DVDSIZE_CUSTOM ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_DVDSIZE_BITRATE, BM_SETCHECK, 
		(config->GetDVDSize() == DVDSIZE_BITRATE ? BST_CHECKED : BST_UNCHECKED), 0);
	SetDlgItemText(hWnd(), IDC_CUSTOMSIZE, String(config->GetCustomSize()).c_str());
	SetDlgItemText(hWnd(), IDC_CUSTOMBITRATE, String(config->GetCustomBitRate()).c_str());
	SendDlgItemMessage(hWnd(), IDC_OUTPUTTYPE_BURNDVD, BM_SETCHECK, 
		(config->GetOutputType() == OUTPUTTYPE_BURNDVD ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_OUTPUTTYPE_ISOIMAGE, BM_SETCHECK, 
		(config->GetOutputType() == OUTPUTTYPE_ISOIMAGE ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_OUTPUTTYPE_DVDFOLDER, BM_SETCHECK, 
		(config->GetOutputType() == OUTPUTTYPE_DVDFOLDER ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_OUTPUTTYPE_MPEGFILES, BM_SETCHECK, 
		(config->GetOutputType() == OUTPUTTYPE_MPEGFILES ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_OUTPUTTYPE_M2VAC3FILES, BM_SETCHECK, 
		(config->GetOutputType() == OUTPUTTYPE_M2VAC3FILES ? BST_CHECKED : BST_UNCHECKED), 0);

	SendDlgItemMessage(hWnd(), IDC_COMPENSATEOVERSCAN, BM_SETCHECK,
		(config->GetCompensateOverscan() ? BST_CHECKED : BST_UNCHECKED), 0);
	SetDlgItemText(hWnd(), IDC_OVERSCANPERCENT, String(config->GetOverscanPercent()).c_str());
	SendDlgItemMessage(hWnd(), IDC_CREATECHAPTERS, BM_SETCHECK,
		(config->GetCreateChapters() ? BST_CHECKED : BST_UNCHECKED), 0);
	SetDlgItemText(hWnd(), IDC_CHAPTERCOUNT, String(config->GetChapterCount()).c_str());
	SetDlgItemText(hWnd(), IDC_SUBFONTFACE, config->GetFontFace().c_str());
	SetDlgItemText(hWnd(), IDC_SUBFONTSIZE, String(config->GetFontSize()).c_str());
	SendDlgItemMessage(hWnd(), IDC_SUBBOLD, BM_SETCHECK,
		(config->GetBold() ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_SUBITALIC, BM_SETCHECK,
		(config->GetItalic() ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_SUBBOX, BM_SETCHECK,
		(config->GetBox() ? BST_CHECKED : BST_UNCHECKED), 0);
	SetWindowLong(GetDlgItem(hWnd(), IDC_VIEWCOLOR), GWL_USERDATA, config->GetColor());
	InvalidateRect(GetDlgItem(hWnd(), IDC_VIEWCOLOR), NULL, FALSE);

	SendDlgItemMessage(hWnd(), IDC_VIDEOOPENER, CB_SETCURSEL, 
		SendDlgItemMessage(hWnd(), IDC_VIDEOOPENER, CB_FINDSTRINGEXACT, 0, (LPARAM) config->GetVideoOpener().c_str()), 0);
	SendDlgItemMessage(hWnd(), IDC_VIDEOPREPROCESSOR, CB_SETCURSEL, 
		SendDlgItemMessage(hWnd(), IDC_VIDEOPREPROCESSOR, CB_FINDSTRINGEXACT, 0, (LPARAM) config->GetVideoPreProcessor().c_str()), 0);
	SendDlgItemMessage(hWnd(), IDC_AUDIOOPENER, CB_SETCURSEL, 
		SendDlgItemMessage(hWnd(), IDC_AUDIOOPENER, CB_FINDSTRINGEXACT, 0, (LPARAM) config->GetAudioOpener().c_str()), 0);
	SendDlgItemMessage(hWnd(), IDC_AUDIOPREPROCESSOR, CB_SETCURSEL, 
		SendDlgItemMessage(hWnd(), IDC_AUDIOPREPROCESSOR, CB_FINDSTRINGEXACT, 0, (LPARAM) config->GetAudioPreProcessor().c_str()), 0);

	SetWindowLong(GetDlgItem(hWnd(), IDC_VIEWBGCOLOR), GWL_USERDATA, config->GetMenuColor(MENUCOLOR_BACKGROUND));
	InvalidateRect(GetDlgItem(hWnd(), IDC_VIEWBGCOLOR), NULL, FALSE);
	SetWindowLong(GetDlgItem(hWnd(), IDC_VIEWNORMALCOLOR), GWL_USERDATA, config->GetMenuColor(MENUCOLOR_NORMAL));
	InvalidateRect(GetDlgItem(hWnd(), IDC_VIEWNORMALCOLOR), NULL, FALSE);
	SetWindowLong(GetDlgItem(hWnd(), IDC_VIEWHILITECOLOR), GWL_USERDATA, config->GetMenuColor(MENUCOLOR_HILITE));
	InvalidateRect(GetDlgItem(hWnd(), IDC_VIEWHILITECOLOR), NULL, FALSE);
	SetWindowLong(GetDlgItem(hWnd(), IDC_VIEWSELECTCOLOR), GWL_USERDATA, config->GetMenuColor(MENUCOLOR_SELECT));
	InvalidateRect(GetDlgItem(hWnd(), IDC_VIEWSELECTCOLOR), NULL, FALSE);
	SendDlgItemMessage(hWnd(), IDC_PLAYTEXTISFILENAME, BM_SETCHECK,
		(config->GetPlayTextFromFileName() ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_SELECTTEXTISLANGUAGE, BM_SETCHECK,
		(config->GetSelectTextFromLanguage() ? BST_CHECKED : BST_UNCHECKED), 0);
	SetDlgItemText(hWnd(), IDC_MENUFONTFACE, config->GetMenuFontFace().c_str());
	SetDlgItemText(hWnd(), IDC_MENUFONTSIZE, String(config->GetMenuFontSize()).c_str());
	SendDlgItemMessage(hWnd(), IDC_MENUBOLD, BM_SETCHECK,
		(config->GetMenuBold() ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_MENUITALIC, BM_SETCHECK,
		(config->GetMenuItalic() ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_NORMALTEXT, BM_SETCHECK,
		(config->GetNormalText() ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_HILITETEXT, BM_SETCHECK,
		(config->GetHiliteText() ? BST_CHECKED : BST_UNCHECKED), 0);
	SetDlgItemText(hWnd(), IDC_HMARGIN, String(config->GetHMargin()).c_str());
	SetDlgItemText(hWnd(), IDC_VMARGIN, String(config->GetVMargin()).c_str());
	SendDlgItemMessage(hWnd(), IDC_AUTORESIZE, BM_SETCHECK,
		(config->GetAutoResize() ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_ALIGNMENT, CB_SETCURSEL, config->GetAlignment(), 0);
	SendDlgItemMessage(hWnd(), IDC_NORMALRECT, BM_SETCHECK,
		(config->GetNormalRect() ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_FILLNORMAL, BM_SETCHECK,
		(config->GetFillNormal() ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_HILITERECT, BM_SETCHECK,
		(config->GetHiliteRect() ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_FILLHILITE, BM_SETCHECK,
		(config->GetFillHilite() ? BST_CHECKED : BST_UNCHECKED), 0);
	SetDlgItemText(hWnd(), IDC_THICKNESS, String(config->GetThickness()).c_str());
	SetDlgItemText(hWnd(), IDC_HRADIUS, String(config->GetHRadius()).c_str());
	SetDlgItemText(hWnd(), IDC_VRADIUS, String(config->GetVRadius()).c_str());	
	
	SendDlgItemMessage(hWnd(), IDC_ENCODER_HC, BM_SETCHECK,
		(config->GetEncoder() == ENCODER_HC ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_ENCODER_QUENC, BM_SETCHECK,
		(config->GetEncoder() == ENCODER_QU ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_HC_PROFILE_FAST, BM_SETCHECK,
		(config->GetHCProfile() == HCPROFILE_FAST ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_HC_PROFILE_NORMAL, BM_SETCHECK,
		(config->GetHCProfile() == HCPROFILE_NORMAL ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_HC_PROFILE_BEST, BM_SETCHECK,
		(config->GetHCProfile() == HCPROFILE_BEST ? BST_CHECKED : BST_UNCHECKED), 0);
	SetDlgItemText(hWnd(), IDC_HCOPTIONS, config->GetHCOptions().crlf().c_str());
	SendDlgItemMessage(hWnd(), IDC_QUENC_2PASS, BM_SETCHECK,
		(config->GetQu2Pass() ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_QUENC_HQ, BM_SETCHECK,
		(config->GetQuHighQuality() ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_QUENC_TRELLIS, BM_SETCHECK,
		(config->GetQuTrellis() ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_QUENC_EXTREME, BM_SETCHECK,
		(config->GetQuExtreme() ? BST_CHECKED : BST_UNCHECKED), 0);
	SetDlgItemText(hWnd(), IDC_QUENCPARAMS, config->GetQuParams().c_str());
	SendDlgItemMessage(hWnd(), IDC_HIDEENCODER, BM_SETCHECK, 
		(config->GetHideEncoder() ? BST_CHECKED : BST_UNCHECKED), 0);
	SetDlgItemText(hWnd(), IDC_AFTENPARAMS, config->GetAftenParams().c_str());
	SetDlgItemText(hWnd(), IDC_IMGBURNEXE, config->GetImgBurn().c_str());
}

void ConfigDialog::Refresh()
{
	SendDlgItemMessage(hWnd(), IDC_VIDEOOPENER, CB_RESETCONTENT, 0, 0);
	SendDlgItemMessage(hWnd(), IDC_AUDIOOPENER, CB_RESETCONTENT, 0, 0);
	for (Config::FunctionMap::const_iterator fi = g_Config->GetOpeners().begin(); 
		fi != g_Config->GetOpeners().end(); fi++) {
		SendDlgItemMessage(hWnd(), IDC_VIDEOOPENER, CB_ADDSTRING, 0, (LPARAM) ((*fi).first.c_str()));
		SendDlgItemMessage(hWnd(), IDC_AUDIOOPENER, CB_ADDSTRING, 0, (LPARAM) ((*fi).first.c_str()));
	}

	SendDlgItemMessage(hWnd(), IDC_VIDEOPREPROCESSOR, CB_RESETCONTENT, 0, 0);
	SendDlgItemMessage(hWnd(), IDC_VIDEOPREPROCESSOR, CB_ADDSTRING, 0, (LPARAM) TEXT(""));
	SendDlgItemMessage(hWnd(), IDC_AUDIOPREPROCESSOR, CB_RESETCONTENT, 0, 0);
	SendDlgItemMessage(hWnd(), IDC_AUDIOPREPROCESSOR, CB_ADDSTRING, 0, (LPARAM) TEXT(""));
	for (Config::FunctionMap::const_iterator fi = g_Config->GetPreProcessors().begin(); 
		fi != g_Config->GetPreProcessors().end(); fi++) {
		SendDlgItemMessage(hWnd(), IDC_VIDEOPREPROCESSOR, CB_ADDSTRING, 0, (LPARAM) ((*fi).first.c_str()));
		SendDlgItemMessage(hWnd(), IDC_AUDIOPREPROCESSOR, CB_ADDSTRING, 0, (LPARAM) ((*fi).first.c_str()));
	}
	
	Update();
}

bool ConfigDialog::DialogProc(UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
	case WM_INITDIALOG:
		InitDialog();
		return true;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {

		case IDC_LOADFROMFILE:
			if (HIWORD(wParam) == BN_CLICKED) {
				TCHAR fileName[MAX_PATH] = TEXT("");
				OPENFILENAME ofn;
				ZeroMemory(&ofn, sizeof(OPENFILENAME));
				ofn.lStructSize     = sizeof(OPENFILENAME);
				ofn.hwndOwner       = hWnd();
				ofn.hInstance       = hInstance;
				ofn.lpstrFilter     = TEXT("DVDforger configuration file\0*.dfc\0\0");
				ofn.lpstrFile       = fileName;
				ofn.nMaxFile        = MAX_PATH;
				ofn.lpstrInitialDir = NULL;
				ofn.lpstrTitle      = TEXT("Load configuration from file");
				ofn.Flags           = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_EXPLORER;
				if (GetOpenFileName(& ofn)) {
					if (! g_Config->Load(String(fileName))) {
						MessageBox(hWnd(), (String(fileName).enclose() + S(" doesn't seem to contain DVDforger configuration data.")).c_str(),
							APPLICATION, MB_OK | MB_ICONSTOP);
					}
					Update();
				}
				return true;
			}
			return false;

		case IDC_SAVETOFILE:
			if (HIWORD(wParam) == BN_CLICKED) {
				TCHAR fileName[MAX_PATH] = TEXT("");
				OPENFILENAME ofn;
				ZeroMemory(&ofn, sizeof(OPENFILENAME));
				ofn.lStructSize     = sizeof(OPENFILENAME);
				ofn.hwndOwner       = hWnd();
				ofn.hInstance       = hInstance;
				ofn.lpstrFilter     = TEXT("DVDforger configuration file\0*.dfc\0\0");
				ofn.lpstrFile       = fileName;
				ofn.nMaxFile        = MAX_PATH;
				ofn.lpstrInitialDir = NULL;
				ofn.lpstrTitle      = TEXT("Save configuration to file");
				ofn.Flags           = OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
				ofn.lpstrDefExt     = TEXT("dfc");
				if (GetSaveFileName(&ofn)) {
					if (! g_Config->Save(String(fileName))) {
						MessageBox(hWnd(), (S("Could not write file ") + String(fileName).enclose()).c_str(),
							APPLICATION, MB_OK | MB_ICONSTOP);
					}
				}
				return true;
			}
			return false;

		case IDC_DVDFORMAT_PAL:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetDVDFormat(DVDFORMAT_PAL);
				Update();
				return true;
			}
			return false;

		case IDC_DVDFORMAT_NTSC:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetDVDFormat(DVDFORMAT_NTSC);
				Update();
				return true;
			}
			return false;

		case IDC_DVDSIZE_DVD_5:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetDVDSize(DVDSIZE_DVD_5);
				Update();
				return true;
			}
			return false;

		case IDC_DVDSIZE_DVD_9:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetDVDSize(DVDSIZE_DVD_9);
				Update();
				return true;
			}
			return false;

		case IDC_DVDSIZE_CUSTOM:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetDVDSize(DVDSIZE_CUSTOM);
				Update();
				return true;
			}
			return false;

		case IDC_DVDSIZE_BITRATE:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetDVDSize(DVDSIZE_BITRATE);
				Update();
				return true;
			}
			return false;

		case IDC_CUSTOMSIZE:
			if (HIWORD(wParam) == EN_KILLFOCUS) {
				int customSize = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_CUSTOMSIZE)).toAnsi().c_str());
				g_Config->SetCustomSize(customSize);
				Update();
				return true;
			}
			return false;

		case IDC_CUSTOMBITRATE:
			if (HIWORD(wParam) == EN_KILLFOCUS) {
				int customBitRate = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_CUSTOMBITRATE)).toAnsi().c_str());
				g_Config->SetCustomBitRate(customBitRate);
				Update();
				return true;
			}
			return false;

		case IDC_OUTPUTTYPE_BURNDVD:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetOutputType(OUTPUTTYPE_BURNDVD);
				Update();
				return true;
			}
			return false;

		case IDC_OUTPUTTYPE_ISOIMAGE:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetOutputType(OUTPUTTYPE_ISOIMAGE);
				Update();
				return true;
			}
			return false;

		case IDC_OUTPUTTYPE_DVDFOLDER:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetOutputType(OUTPUTTYPE_DVDFOLDER);
				Update();
				return true;
			}
			return false;

		case IDC_OUTPUTTYPE_MPEGFILES:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetOutputType(OUTPUTTYPE_MPEGFILES);
				Update();
				return true;
			}
			return false;

		case IDC_OUTPUTTYPE_M2VAC3FILES:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetOutputType(OUTPUTTYPE_M2VAC3FILES);
				Update();
				return true;
			}
			return false;

		case IDC_COMPENSATEOVERSCAN:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetCompensateOverscan(! g_Config->GetCompensateOverscan());
				Update();
				return true;
			}
			return false;

		case IDC_OVERSCANPERCENT:
			if (HIWORD(wParam) == EN_KILLFOCUS) {
				int overscanPercent = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_OVERSCANPERCENT)).toAnsi().c_str());
				g_Config->SetOverscanPercent(overscanPercent);
				Update();
				return true;
			}
			return false;

		case IDC_CREATECHAPTERS:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetCreateChapters(! g_Config->GetCreateChapters());
				Update();
				return true;
			}
			return false;

		case IDC_CHAPTERCOUNT:
			if (HIWORD(wParam) == EN_KILLFOCUS) {
				int chapterCount = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_CHAPTERCOUNT)).toAnsi().c_str());
				g_Config->SetChapterCount(chapterCount);
				Update();
				return true;
			}
			return false;

		case IDC_SUBFONTFACE:
			if (HIWORD(wParam) == CBN_KILLFOCUS) {
				String fontFace = String().fromWindow(GetDlgItem(hWnd(), IDC_SUBFONTFACE));
				g_Config->SetFontFace(fontFace);
				Update();
				return true;
			}
			return false;

		case IDC_SUBFONTSIZE:
			if (HIWORD(wParam) == EN_KILLFOCUS) {
				int fontSize = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_SUBFONTSIZE)).toAnsi().c_str());
				g_Config->SetFontSize(fontSize);
				Update();
				return true;
			}
			return false;

		case IDC_SUBBOLD:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetBold(! g_Config->GetBold());
				Update();
				return true;
			}
			return false;

		case IDC_SUBITALIC:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetItalic(! g_Config->GetItalic());
				Update();
				return true;
			}
			return false;

		case IDC_SUBBOX:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetBox(! g_Config->GetBox());
				Update();
				return true;
			}
			return false;

		case IDC_CHOOSECOLOR:
			if (HIWORD(wParam) == BN_CLICKED) {
				ChooseAColor(-1);
				return true;
			}
			return false;

		case IDC_EDITAVSFUNCTIONS:
			if (HIWORD(wParam) == BN_CLICKED) {
				ShellExecute(hWnd(), NULL, g_Config->GetFunctionDir().c_str(), NULL, NULL, SW_SHOWNORMAL);
				return true;
			}
			return false;

		case IDC_VIDEOOPENER:
			if (HIWORD(wParam) == CBN_KILLFOCUS) {
				String value;
				int index = SendDlgItemMessage(hWnd(), IDC_VIDEOOPENER, CB_GETCURSEL, 0, 0);
				if (index >= 0) {
					int length = (int) SendDlgItemMessage(hWnd(), IDC_VIDEOOPENER, CB_GETLBTEXTLEN, index, 0);
					LPTSTR s = new TCHAR[length + 1];
					SendDlgItemMessage(hWnd(), IDC_VIDEOOPENER, CB_GETLBTEXT, index, (LPARAM) s);
					value = s;
					delete [] s;
				}
				g_Config->SetVideoOpener(value);
				return true;
			}
			return false;

		case IDC_VIDEOPREPROCESSOR:
			if (HIWORD(wParam) == CBN_KILLFOCUS) {
				String value;
				int index = SendDlgItemMessage(hWnd(), IDC_VIDEOPREPROCESSOR, CB_GETCURSEL, 0, 0);
				if (index >= 0) {
					int length = (int) SendDlgItemMessage(hWnd(), IDC_VIDEOPREPROCESSOR, CB_GETLBTEXTLEN, index, 0);
					LPTSTR s = new TCHAR[length + 1];
					SendDlgItemMessage(hWnd(), IDC_VIDEOPREPROCESSOR, CB_GETLBTEXT, index, (LPARAM) s);
					value = s;
					delete [] s;
				}
				g_Config->SetVideoPreProcessor(value);
				return true;
			}
			return false;

		case IDC_AUDIOOPENER:
			if (HIWORD(wParam) == CBN_KILLFOCUS) {
				String value;
				int index = SendDlgItemMessage(hWnd(), IDC_AUDIOOPENER, CB_GETCURSEL, 0, 0);
				if (index >= 0) {
					int length = (int) SendDlgItemMessage(hWnd(), IDC_AUDIOOPENER, CB_GETLBTEXTLEN, index, 0);
					LPTSTR s = new TCHAR[length + 1];
					SendDlgItemMessage(hWnd(), IDC_AUDIOOPENER, CB_GETLBTEXT, index, (LPARAM) s);
					value = s;
					delete [] s;
				}
				g_Config->SetAudioOpener(value);
				return true;
			}
			return false;

		case IDC_AUDIOPREPROCESSOR:
			if (HIWORD(wParam) == CBN_KILLFOCUS) {
				String value;
				int index = SendDlgItemMessage(hWnd(), IDC_AUDIOPREPROCESSOR, CB_GETCURSEL, 0, 0);
				if (index >= 0) {
					int length = (int) SendDlgItemMessage(hWnd(), IDC_AUDIOPREPROCESSOR, CB_GETLBTEXTLEN, index, 0);
					LPTSTR s = new TCHAR[length + 1];
					SendDlgItemMessage(hWnd(), IDC_AUDIOPREPROCESSOR, CB_GETLBTEXT, index, (LPARAM) s);
					value = s;
					delete [] s;
				}
				g_Config->SetAudioPreProcessor(value);
				return true;
			}
			return false;

		case IDC_CHOOSEBGCOLOR:
			if (HIWORD(wParam) == BN_CLICKED) {
				ChooseAColor(MENUCOLOR_BACKGROUND);
				return true;
			}
			return false;

		case IDC_CHOOSENORMALCOLOR:
			if (HIWORD(wParam) == BN_CLICKED) {
				ChooseAColor(MENUCOLOR_NORMAL);
				return true;
			}
			return false;

		case IDC_CHOOSEHILITECOLOR:
			if (HIWORD(wParam) == BN_CLICKED) {
				ChooseAColor(MENUCOLOR_HILITE);
				return true;
			}
			return false;

		case IDC_CHOOSESELECTCOLOR:
			if (HIWORD(wParam) == BN_CLICKED) {
				ChooseAColor(MENUCOLOR_SELECT);
				return true;
			}
			return false;

		case IDC_PLAYTEXTISFILENAME:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetPlayTextFromFileName(! g_Config->GetPlayTextFromFileName());
				Update();
				return true;
			}
			return false;

		case IDC_SELECTTEXTISLANGUAGE:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetSelectTextFromLanguage(! g_Config->GetSelectTextFromLanguage());
				Update();
				return true;
			}
			return false;

		case IDC_MENUFONTFACE:
			if (HIWORD(wParam) == CBN_KILLFOCUS) {
				String fontFace = String().fromWindow(GetDlgItem(hWnd(), IDC_MENUFONTFACE));
				g_Config->SetMenuFontFace(fontFace);
				Update();
				return true;
			}
			return false;

		case IDC_MENUFONTSIZE:
			if (HIWORD(wParam) == EN_KILLFOCUS) {
				int fontSize = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_MENUFONTSIZE)).toAnsi().c_str());
				g_Config->SetMenuFontSize(fontSize);
				Update();
				return true;
			}
			return false;

		case IDC_MENUBOLD:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetMenuBold(! g_Config->GetMenuBold());
				Update();
				return true;
			}
			return false;

		case IDC_MENUITALIC:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetMenuItalic(! g_Config->GetMenuItalic());
				Update();
				return true;
			}
			return false;

		case IDC_NORMALTEXT:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetNormalText(! g_Config->GetNormalText());
				Update();
				return true;
			}
			return false;

		case IDC_HILITETEXT:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetHiliteText(! g_Config->GetHiliteText());
				Update();
				return true;
			}
			return false;

		case IDC_HMARGIN:
			if (HIWORD(wParam) == EN_KILLFOCUS) {
				int margin = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_HMARGIN)).toAnsi().c_str());
				g_Config->SetHMargin(margin);
				Update();
				return true;
			}
			return false;

		case IDC_VMARGIN:
			if (HIWORD(wParam) == EN_KILLFOCUS) {
				int margin = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_VMARGIN)).toAnsi().c_str());
				g_Config->SetVMargin(margin);
				Update();
				return true;
			}
			return false;

		case IDC_AUTORESIZE:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetAutoResize(! g_Config->GetAutoResize());
				Update();
				return true;
			}
			return false;

		case IDC_ALIGNMENT:
			if (HIWORD(wParam) == CBN_KILLFOCUS) {
				int alignment = SendDlgItemMessage(hWnd(), IDC_ALIGNMENT, CB_GETCURSEL, 0, 0);
				g_Config->SetAlignment(alignment);
				Update();
				return true;
			}
			return false;

		case IDC_NORMALRECT:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetNormalRect(! g_Config->GetNormalRect());
				Update();
				return true;
			}
			return false;

		case IDC_FILLNORMAL:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetFillNormal(! g_Config->GetFillNormal());
				Update();
				return true;
			}
			return false;

		case IDC_HILITERECT:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetHiliteRect(! g_Config->GetHiliteRect());
				Update();
				return true;
			}
			return false;

		case IDC_FILLHILITE:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetFillHilite(! g_Config->GetFillHilite());
				Update();
				return true;
			}
			return false;

		case IDC_THICKNESS:
			if (HIWORD(wParam) == EN_KILLFOCUS) {
				int thickness = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_THICKNESS)).toAnsi().c_str());
				g_Config->SetThickness(thickness);
				Update();
				return true;
			}
			return false;

		case IDC_HRADIUS:
			if (HIWORD(wParam) == EN_KILLFOCUS) {
				int radius = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_HRADIUS)).toAnsi().c_str());
				g_Config->SetHRadius(radius);
				Update();
				return true;
			}
			return false;

		case IDC_VRADIUS:
			if (HIWORD(wParam) == EN_KILLFOCUS) {
				int radius = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_VRADIUS)).toAnsi().c_str());
				g_Config->SetVRadius(radius);
				Update();
				return true;
			}
			return false;

		case IDC_ENCODER_HC:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetEncoder(ENCODER_HC);
				Update();
				return true;
			}
			return false;

		case IDC_ENCODER_QUENC:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetEncoder(ENCODER_QU);
				Update();
				return true;
			}
			return false;

		case IDC_HC_PROFILE_FAST:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetHCProfile(HCPROFILE_FAST);
				Update();
				return true;
			}
			return false;

		case IDC_HC_PROFILE_NORMAL:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetHCProfile(HCPROFILE_NORMAL);
				Update();
				return true;
			}
			return false;

		case IDC_HC_PROFILE_BEST:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetHCProfile(HCPROFILE_BEST);
				Update();
				return true;
			}
			return false;

		case IDC_HCOPTIONS:
			if (HIWORD(wParam) == EN_KILLFOCUS) {
				String options = String().fromWindow(GetDlgItem(hWnd(), IDC_HCOPTIONS)).lfOnly();
				g_Config->SetHCOptions(options);
				Update();
				return true;
			}
			return false;

		case IDC_QUENC_2PASS:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetQu2Pass(! g_Config->GetQu2Pass());
				Update();
				return true;
			}
			return false;

		case IDC_QUENC_HQ:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetQuHighQuality(! g_Config->GetQuHighQuality());
				Update();
				return true;
			}
			return false;

		case IDC_QUENC_TRELLIS:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetQuTrellis(! g_Config->GetQuTrellis());
				Update();
				return true;
			}
			return false;

		case IDC_QUENC_EXTREME:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetQuExtreme(! g_Config->GetQuExtreme());
				Update();
				return true;
			}
			return false;

		case IDC_QUENCPARAMS:
			if (HIWORD(wParam) == EN_KILLFOCUS) {
				String params = String().fromWindow(GetDlgItem(hWnd(), IDC_QUENCPARAMS));
				g_Config->SetQuParams(params);
				Update();
				return true;
			}
			return false;

		case IDC_HIDEENCODER:
			if (HIWORD(wParam) == BN_CLICKED) {
				g_Config->SetHideEncoder(! g_Config->GetHideEncoder());
				Update();
				return true;
			}
			return false;

		case IDC_AFTENPARAMS:
			if (HIWORD(wParam) == EN_KILLFOCUS) {
				String params = String().fromWindow(GetDlgItem(hWnd(), IDC_AFTENPARAMS));
				g_Config->SetAftenParams(params);
				Update();
				return true;
			}
			return false;

		case IDC_BROWSEIMGBURN:
			if (HIWORD(wParam) == BN_CLICKED) {
				TCHAR fileName[MAX_PATH];
				lstrcpyn(fileName, g_Config->GetImgBurn().c_str(), MAX_PATH);
				OPENFILENAME ofn;
				ZeroMemory(&ofn, sizeof(OPENFILENAME));
				ofn.lStructSize     = sizeof(OPENFILENAME);
				ofn.hwndOwner       = hWnd();
				ofn.hInstance       = hInstance;
				ofn.lpstrFilter     = TEXT("EXE files\0*.exe\0\0");
				ofn.lpstrFile       = fileName;
				ofn.nMaxFile        = MAX_PATH;
				ofn.lpstrInitialDir = NULL;
				ofn.lpstrTitle      = TEXT("Browse ImgBurn.exe");
				ofn.Flags           = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_EXPLORER;
				if (GetOpenFileName(& ofn)) {
					g_Config->SetImgBurn(String(fileName));
					Update();
				}
				return true;
			}
			return false;

		case IDC_BACK:
			if (HIWORD(wParam) == BN_CLICKED) {
				SendMessage(hWnd(), WM_CLOSE, 0, 0);
				return true;
			}
			return false;

		default:
			return false;
		}

	case WM_CLOSE:
		if (g_Config->HasChanged()) {
			if (g_Config->Save()) {
				g_Config->ClearChanged();
			}
		}
		EndDialog(hWnd(), 0);
		return true;
	}
	return false;
}

int CALLBACK InitConfigFonts(LPENUMLOGFONT Font, LPNEWTEXTMETRIC Metric, int FontType, LPARAM lParam)
{
	HWND hWnd;
	hWnd = (HWND) lParam;
	SendDlgItemMessage(hWnd, IDC_SUBFONTFACE, CB_ADDSTRING, 0, (LPARAM)(Font->elfLogFont.lfFaceName));
	SendDlgItemMessage(hWnd, IDC_MENUFONTFACE, CB_ADDSTRING, 0, (LPARAM)(Font->elfLogFont.lfFaceName));
	return 1;
}

void ConfigDialog::InitDialog()
{
	HDC dc = GetDC(hWnd());
	EnumFontFamilies(dc, NULL, (FONTENUMPROC) InitConfigFonts, (LPARAM) hWnd());
	ReleaseDC(hWnd(), dc);

	SendDlgItemMessage(hWnd(), IDC_ALIGNMENT, CB_ADDSTRING, 0, (LPARAM) TEXT("Top-left"));
	SendDlgItemMessage(hWnd(), IDC_ALIGNMENT, CB_ADDSTRING, 0, (LPARAM) TEXT("Top-center"));
	SendDlgItemMessage(hWnd(), IDC_ALIGNMENT, CB_ADDSTRING, 0, (LPARAM) TEXT("Top-right"));
	SendDlgItemMessage(hWnd(), IDC_ALIGNMENT, CB_ADDSTRING, 0, (LPARAM) TEXT("Center-left"));
	SendDlgItemMessage(hWnd(), IDC_ALIGNMENT, CB_ADDSTRING, 0, (LPARAM) TEXT("Center"));
	SendDlgItemMessage(hWnd(), IDC_ALIGNMENT, CB_ADDSTRING, 0, (LPARAM) TEXT("Center-right"));
	SendDlgItemMessage(hWnd(), IDC_ALIGNMENT, CB_ADDSTRING, 0, (LPARAM) TEXT("Bottom-left"));
	SendDlgItemMessage(hWnd(), IDC_ALIGNMENT, CB_ADDSTRING, 0, (LPARAM) TEXT("Bottom-center"));
	SendDlgItemMessage(hWnd(), IDC_ALIGNMENT, CB_ADDSTRING, 0, (LPARAM) TEXT("Bottom-right"));

	Refresh();
}

void ConfigDialog::ChooseAColor(int which)
{
	CHOOSECOLOR cc;
	cc.lStructSize    = sizeof(CHOOSECOLOR);
	cc.hwndOwner      = hWnd();
	cc.hInstance      = NULL;
	cc.lpCustColors   = CustomColors;
	cc.Flags          = CC_ANYCOLOR | CC_FULLOPEN | CC_RGBINIT;
	cc.lCustData      = 0;
	cc.lpfnHook       = NULL;
	cc.lpTemplateName = NULL;
	if (which < 0) {
		cc.rgbResult = g_Config->GetColor();
	} else {
		cc.rgbResult = g_Config->GetMenuColor(which);
	}
	if (ChooseColor(&cc)) {
		if (which < 0) {
			g_Config->SetColor(cc.rgbResult);
		} else {
			g_Config->SetMenuColor(which, cc.rgbResult);
		}
		Update();
	}
}
