#include "dialog.h"

Dialog::Dialog()
{
	m_hWnd = 0;
	m_IsModal = false;
}

Dialog::~Dialog()
{
	if (m_hWnd) {
		SetWindowLong(m_hWnd, DWL_USER, 0);
		if (m_IsModal) {
			EndDialog(m_hWnd, 0);
		} else {
			DestroyWindow(m_hWnd);
		}
	}
}

bool Dialog::Create(HINSTANCE hInstance, LPCTSTR lpTemplate, HWND hWndParent)
{
	if (m_hWnd) return false;
	m_hWnd = CreateDialogParam(hInstance, lpTemplate, hWndParent, Dialog::_DlgProc, (LPARAM) this);
	if (! m_hWnd) return false;
	return true;
}

int Dialog::DoModal(HINSTANCE hInstance, LPCTSTR lpTemplate, HWND hWndParent)
{
	if (m_hWnd) return false;
	m_IsModal = true;
	int result = DialogBoxParam(hInstance, lpTemplate, hWndParent, Dialog::_DlgProc, (LPARAM) this);
	return result;
}

BOOL CALLBACK Dialog::_DlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	BOOL result = FALSE;
	Dialog * dialog = 0;
	if (message == WM_INITDIALOG) {
		SetWindowLong(hWnd, DWL_USER, (LONG) lParam);
		dialog = (Dialog *) lParam;
		dialog->m_hWnd = hWnd;
		dialog->m_PrevDialog = s_ActDialog;
		s_ActDialog = dialog;
		SendMessage(hWnd, WM_SETFONT, (WPARAM) GetStockObject(DEFAULT_GUI_FONT), TRUE);
	} else {
		dialog = (Dialog *) GetWindowLong(hWnd, DWL_USER);
	}
	if (dialog) {
		result = dialog->DialogProc(message, wParam, lParam);
		if (message == WM_DESTROY) {
			dialog->m_hWnd = 0;
			dialog->m_IsModal = false;
			s_ActDialog = dialog->m_PrevDialog;
		}
	}
	return result;
}

Dialog * Dialog::s_ActDialog = NULL;

Dialog * Dialog::GetActDialog()
{
	return s_ActDialog;
}

void Dialog::Refresh()
{
}