#include "drawitem.h"

#define BORDER 2

void MeasureListItem(HWND hWnd, LPMEASUREITEMSTRUCT mis, StringVectorVector & strings, int indent)
{
	int prefixWidth = 0;
	int maxWidth    = 0;
	int sumHeight   = 0;

	HDC dc = GetDC(hWnd);
	HFONT font = (HFONT) SendMessage(hWnd, WM_GETFONT, 0, 0);
	HFONT origFont = (HFONT) SelectObject(dc, font);

	if (indent >= 0) {
		SIZE size;
		GetTextExtentPoint32(dc, TEXT("1"), 1, & size);
		prefixWidth = size.cy * (indent ? indent : 2);
	}
	for (StringVectorVector::iterator vectorIterator = strings.begin(); 
		vectorIterator != strings.end(); vectorIterator++) {
		int lineWidth  = prefixWidth;
		int lineHeight = 0;
		if (! (*vectorIterator).size()) {
			lineHeight = BORDER * 2 + 1;
		} else for (StringVector::iterator stringIterator = (*vectorIterator).begin();
			stringIterator != (*vectorIterator).end(); stringIterator++) {
			if ((*stringIterator).length()) {
				SIZE size;
				GetTextExtentPoint32(dc, (*stringIterator).c_str(), (int) (*stringIterator).length(), & size);
				lineWidth += size.cx;
				if (lineHeight < size.cy) lineHeight = size.cy;
			}
		}
		if (maxWidth < lineWidth) maxWidth = lineWidth;
		sumHeight += lineHeight;
	}

	SelectObject(dc, origFont);
	ReleaseDC(hWnd, dc);

	mis->itemWidth  = maxWidth + 2 * BORDER;
	mis->itemHeight = sumHeight + 2 * BORDER + 1;
}

void DrawListItem(HWND hWnd, LPDRAWITEMSTRUCT dis, StringVectorVector & strings, 
				  int index, int indent, int nextindent, bool overridesel)
{
	RECT rect;
	rect.left   = 0;
	rect.top    = 0;
	rect.right  = dis->rcItem.right - dis->rcItem.left;
	rect.bottom = dis->rcItem.bottom - dis->rcItem.top;
	HDC dc = CreateCompatibleDC(dis->hDC);
	HBITMAP bitmap = CreateCompatibleBitmap(dis->hDC, rect.right, rect.bottom);
	SelectObject(dc, bitmap);
	FillRect(dc, & rect, GetSysColorBrush(COLOR_WINDOW));
	COLORREF color[2];
	if ((dis->itemState & ODS_SELECTED) != 0 || overridesel) {
		color[0] = GetSysColor(COLOR_HIGHLIGHTTEXT);
		color[1] = GetSysColor(COLOR_HIGHLIGHT);
		SetTextColor(dc, GetSysColor(COLOR_HIGHLIGHTTEXT));
	} else {
		color[0] = GetSysColor(COLOR_WINDOWTEXT);
		color[1] = GetSysColor(COLOR_WINDOW);
		SetTextColor(dc, GetSysColor(COLOR_INFOTEXT));
	}
	HFONT font[2];
	if (index) {
		SelectObject(dc, (HFONT) SendMessage(hWnd, WM_GETFONT, 0, 0));
	} else {
		LOGFONT logFont;
		GetObject((HFONT) SendMessage(hWnd, WM_GETFONT, 0, 0), sizeof(LOGFONT), & logFont);
		logFont.lfWeight = FW_NORMAL;
		font[0] = CreateFontIndirect(& logFont);
		logFont.lfWeight = FW_BOLD;
		font[1] = CreateFontIndirect(& logFont);
		SelectObject(dc, font[0]);
	}
	String indexStr(index);
	SIZE size;
	GetTextExtentPoint32(dc, indexStr.c_str(), 1, & size);
	int prefixBase  = size.cy;
	int prefixWidth = prefixBase;
	if (index > 0) {
		prefixWidth *= 2;
	} else {
		prefixWidth *= indent;
		rect.left += prefixWidth;
	}
	SetBkColor(dc, color[1]);
	HBRUSH brush = CreateSolidBrush(color[1]);
	FillRect(dc, & rect, brush);
	DeleteObject(brush);
	if (index > 0) {
		TextOut(dc, BORDER + (prefixWidth - size.cx) / 2, BORDER, indexStr.c_str(), (int) indexStr.length());
	}
	if ((dis->itemState & ODS_SELECTED) != 0 || overridesel) {
		color[1] = RGB(
			GetRValue(color[1]) + ((GetRValue(color[0]) - GetRValue(color[1])) * 4 / 5),
			GetGValue(color[1]) + ((GetGValue(color[0]) - GetGValue(color[1])) * 4 / 5),
			GetBValue(color[1]) + ((GetBValue(color[0]) - GetBValue(color[1])) * 4 / 5));
	} else {
		color[1] = GetSysColor(COLOR_GRAYTEXT);
	}
	int y = BORDER;
	int hang = 0;
	bool firstLine = true;
	for (StringVectorVector::iterator vectorIterator = strings.begin(); 
		vectorIterator != strings.end(); vectorIterator++) {
		int x = BORDER + prefixWidth;
		int c = 1;
		int height = 0;
		bool firstString = true;
		for (StringVector::iterator stringIterator = (*vectorIterator).begin();
			stringIterator != (*vectorIterator).end(); stringIterator++) {
			if ((*stringIterator).length()) {
				SetTextColor(dc, color[c]);
				if (! index) SelectObject(dc, font[c]);
				SIZE size;
				GetTextExtentPoint32(dc, (*stringIterator).c_str(), (int) (*stringIterator).length(), & size);
				if (firstString && firstLine) {
					hang = size.cx;
				}
				TextOut(dc, x, y, (*stringIterator).c_str(), (int) (*stringIterator).length());
				x += size.cx;
				if (height < size.cy) height = size.cy;
			} else if (! firstLine) {
				x += hang;
			}
			c = 1 - c;
			firstString = false;
		}
		y += height;
		firstLine = false;
	}
	if (! index) {
		DeleteObject(font[1]);
		DeleteObject(font[0]);
	}
	rect.bottom--;
	HPEN pen = CreatePen(PS_SOLID, 1, GetSysColor(COLOR_GRAYTEXT));
	SelectObject(dc, pen);
	MoveToEx(dc, prefixBase * (indent < nextindent ? indent : nextindent) - 1, rect.bottom, NULL);
	LineTo(dc, rect.right, rect.bottom);
	if (indent) {
		MoveToEx(dc, prefixBase * indent - 1, rect.top, NULL);
		LineTo(dc, prefixBase * indent - 1, rect.bottom);
	}
	DeleteObject(pen);
	if ((dis->itemState & ODS_FOCUS)) {
		DrawFocusRect(dc, & rect);
	}
	BitBlt(dis->hDC, dis->rcItem.left, dis->rcItem.top,
		dis->rcItem.right - dis->rcItem.left, dis->rcItem.bottom - dis->rcItem.top,
		dc, 0, 0, SRCCOPY);
	DeleteObject(bitmap);
	DeleteDC(dc);
}
