#ifndef GROUP_H_
#define GROUP_H_

#include "managableptr.h"
#include "managedptr.h"
#include "string.h"

#include "title.h"
#include "video.h"
#include "menu.h"

#include <vector>

#define MENUTYPE_NONE     0
#define MENUTYPE_COMBINED 1
#define MENUTYPE_SEPARATE 2
#define MENUTYPE_MAX      3

#define VIEW_TITLETRACKPART 0
#define VIEW_TRACKTITLEPART 1
#define VIEW_OUTLINE        2
#define VIEW_MAX            3

class Project;

class Group : public ManagablePtr
{
public:
	Group(Project * project);
	virtual ~Group();

public:
	Project * GetProject() const;

	bool SetFirstSubtitleOn(bool firstSubtitleOn, bool setChanged = true);
	bool GetFirstSubtitleOn() const;
	int GetMenuAspectRatio() const;
	String GetMenuAspectRatioStr() const;
	bool SetMenuAspectRatio(int aspectRatio, bool setChanged = true);

	bool CanAddVideo() const;
	bool AddVideo(VideoPtr & video, bool setChanged = true);
	bool IsVideoIndexValid(int index) const;
	int GetVideoCount() const;
	VideoPtr GetVideo(int index) const;
	bool CanMoveSelectedVideosUp() const;
	bool MoveSelectedVideosUp(bool setChanged = true);
	bool CanMoveSelectedVideosDown() const;
	bool MoveSelectedVideosDown(bool setChanged = true);
	bool CanMergeSelectedVideos() const;
	bool MergeSelectedVideos(bool setChanged = true);
	bool CanSplitSelectedVideos() const;
	bool SplitSelectedVideos(bool setChanged = true);
	bool CanDeleteSelectedVideos() const;
	bool DeleteSelectedVideos(bool setChanged = true);

	bool CanAddAudio() const;
	bool AddAudio(int language, bool setChanged = true);
	bool IsAudioIndexValid(int index) const;
	int GetAudioCount() const;
	bool SetAudioLanguage(int index, int language, bool setChanged = true);
	int  GetAudioLanguage(int index) const;
	bool CanMoveAudioUp(int index) const;
	bool MoveAudioUp(int index, bool setChanged = true);
	bool CanMoveAudioDown(int index) const;
	bool MoveAudioDown(int index, bool setChanged = true);
	bool CanDeleteAudio(int index) const;
	bool DeleteAudio(int index, bool setChanged = true);
	int  GetSelectedAudio() const;
	bool SetSelectedAudio(int selectedAudio);
	bool IsAudioSelected() const;

	bool CanAddSubtitle() const;
	bool AddSubtitle(int language, bool setChanged = true);
	bool IsSubtitleIndexValid(int index) const;
	int GetSubtitleCount() const;
	bool SetSubtitleLanguage(int index, int language, bool setChanged = true);
	int  GetSubtitleLanguage(int index) const;
	bool CanMoveSubtitleUp(int index) const;
	bool MoveSubtitleUp(int index, bool setChanged = true);
	bool CanMoveSubtitleDown(int index) const;
	bool MoveSubtitleDown(int index, bool setChanged = true);
	bool CanDeleteSubtitle(int index) const;
	bool DeleteSubtitle(int index, bool setChanged = true);
	int  GetSelectedSubtitle() const;
	bool SetSelectedSubtitle(int selectedSubtitle);
	bool IsSubtitleSelected() const;

	bool HasMenu() const;
	bool SetMenu(bool hasMenu, bool setChanged = true);
	MenuPtr GetMenu() const;
	int  GetMenuType() const;
	bool SetMenuType(int menuType, bool setChanged = true);
	MenuPtr GetCombinedMenu() const;
	bool HasAudioMenu() const;
	bool SetAudioMenu(bool hasAudioMenu, bool setChanged = true);
	MenuPtr GetAudioMenu() const;
	bool HasSubtitleMenu() const;
	bool SetSubtitleMenu(bool hasSubtitleMenu, bool setChanged = true);
	MenuPtr GetSubtitleMenu() const;

	void SetChanged();
	int GetIndex() const;
	void SetIndex(int index);
	String GetIdent();
	String Recalculate();

	int GetTitleCount() const;
	bool IsTitleIndexValid(int index) const;
	TitlePtr GetTitle(int index) const;

	int GetView() const;
	void SetView(int view);
	bool IsVideoTrackSelected() const;
	void SetVideoTrackSelected(bool selected);
	bool IsAudioTrackSelected(int index) const;
	void SetAudioTrackSelected(int index, bool selected);
	bool IsSubtitleTrackSelected(int index) const;
	void SetSubtitleTrackSelected(int index, bool selected);
public:
	void GetDescription(StringVectorVector & strings) const;
	String GetShortDescr(int index) const;
	double GetLength() const;
	int GetSubStreamRate() const;

public:
	bool Load(csv_file_t csv, int & lineno, int nest);
	void Save(FILE * f, int nest) const;

protected:
	Project * m_Project;
	bool m_FirstSubtitleOn;
	int m_MenuAspectRatio;
	std::vector<int> m_AudioLanguages;
	std::vector<bool> m_AudioSelected;
	std::vector<int> m_SubtitleLanguages;
	std::vector<bool> m_SubtitleSelected;
	bool m_VideoTrackSelected;
	VideoVector m_Videos;
	bool m_HasMenu;
	MenuPtr m_Menu;
	int m_MenuType;
	MenuPtr m_CombinedMenu;
	bool m_HasAudioMenu;
	MenuPtr m_AudioMenu;
	bool m_HasSubtitleMenu;
	MenuPtr m_SubtitleMenu;
	double m_Length;
	bool m_IsTitleSelected;
	int m_SelectedVideoCount;
	int m_FirstSelectedVideo;
	int m_LastSelectedVideo;
	int m_SelectedAudio;
	int m_SelectedSubtitle;
	int m_SubStreamRate;
	TitleVector m_Titles;
	int m_View;
	int m_Index;
};

typedef ManagedPtr<Group> GroupPtr;
typedef std::vector<GroupPtr> GroupVector;

#endif // GROUP_H_
