#include "lookatvideodialog.h"
#include "main.h"
#include "resource.h"
#include "EasyBMP/EasyBMP.h"
#include "EasyBMP/EasyBMP_win32.h"
#include "winutils/winutils.h"

LookAtVideoDialog::LookAtVideoDialog(Dialog * parent, const String & fileName, const String & opener, bool upsideDown, const String & preProcessor, bool pal) : Dialog()
{
	m_Parent       = parent;
	m_FileName     = fileName;
	m_Opener       = opener;
	m_UpsideDown   = upsideDown;
	m_PreProcessor = preProcessor;
	m_PAL          = pal;
	m_FirstFrame   = 0;
	m_LastFrame    = -1;
	ZeroMemory(m_FullBitmap, sizeof(HBITMAP) * FRAME_COUNT);
	m_HalfFrames = false;
}

LookAtVideoDialog::~LookAtVideoDialog()
{
	for (int i = 0; i < FRAME_COUNT; i++) {
		if (m_FullBitmap[i]) DeleteObject(m_FullBitmap[i]);
	}
}

int LookAtVideoDialog::DoModal()
{
	return Dialog::DoModal(hInstance, MAKEINTRESOURCE(IDD_LOOKATVIDEO), m_Parent->hWnd());
}

bool LookAtVideoDialog::DialogProc(UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) 
	{
		case WM_INITDIALOG:
		{
			SetWindowText(hWnd(), m_FileName.c_str());
			SetWindowLong(GetDlgItem(hWnd(), IDC_VIEWFRAME), GWL_USERDATA, (LONG) this);
			SendDlgItemMessage(hWnd(), IDC_FULLFRAMES, BM_SETCHECK, BST_CHECKED, 0);
			HCURSOR origCursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
			m_AVS.Open(m_FileName, m_Opener, m_UpsideDown, m_PreProcessor, m_PAL);
			SetCursor(origCursor);
			if (m_AVS.GetInfo().hasVideo) {
				SendMessage(hWnd(), WM_COMMAND, MAKEWPARAM(IDC_LOADFRAMES, BN_CLICKED), 0);
			} else {
				MessageBox(hWnd(), TEXT("Could not load video file."), APPLICATION, MB_ICONSTOP | MB_OK);
				EndDialog(hWnd(), 0);
			}
			return true;
		}

		case WM_COMMAND:
		{
			switch (LOWORD(wParam))
			{
			case IDC_SKIPFRAMES:
				if (HIWORD(wParam) == BN_CLICKED) {
					const AVSWrapper::Info & info = m_AVS.GetInfo();
					if (m_LastFrame < info.frameCount - 901) {
						HCURSOR origCursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
						for (int i = 0; i < 900; i++) {
							HBITMAP bitmap = m_AVS.GetNextFrame();
							if (bitmap) {
								DeleteObject(bitmap);
								m_LastFrame++;
							}
						}
						SendMessage(hWnd(), WM_COMMAND, MAKEWPARAM(IDC_LOADFRAMES, BN_CLICKED), 0);
						SetCursor(origCursor);
					}
					return true;
				}
				return false;

			case IDC_LOADFRAMES:
				if (HIWORD(wParam) == BN_CLICKED) {
					const AVSWrapper::Info & info = m_AVS.GetInfo();
					if (m_LastFrame < info.frameCount - 1) {
						HCURSOR origCursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
						m_FirstFrame = m_LastFrame + 1;
						bool go = true;
						for (int i = 0; go && (i < FRAME_COUNT) && (m_FirstFrame + i < info.frameCount); i++) {
							m_LastFrame = m_FirstFrame + i;
							if (m_FullBitmap[i]) DeleteObject(m_FullBitmap[i]);
							m_FullBitmap[i] = m_AVS.GetNextFrame();
							go = (m_FullBitmap[i] != NULL);
						}
						SetCursor(origCursor);
					}
					EnableWindow(GetDlgItem(hWnd(), IDC_LOADFRAMES), (m_LastFrame < m_AVS.GetInfo().frameCount - 1));
					EnableWindow(GetDlgItem(hWnd(), IDC_SKIPFRAMES), (m_LastFrame < m_AVS.GetInfo().frameCount - 1000));
					SetDlgItemText(hWnd(), IDC_FRAMESINLIST, 
						(S("Frame ") + String(m_FirstFrame + 1) + S(" to ") + String(m_LastFrame + 1) 
						+ S(" of ") + String(info.frameCount)).c_str());
					SendDlgItemMessage(hWnd(), IDC_FRAMELIST, LB_RESETCONTENT, 0, 0);
					for (int i = 0; i < (m_LastFrame - m_FirstFrame + 1) * (m_HalfFrames ? 2 : 1); i++) {
						SendDlgItemMessage(hWnd(), IDC_FRAMELIST, LB_ADDSTRING, 0, i);
					}
					SendDlgItemMessage(hWnd(), IDC_FRAMELIST, LB_SETCURSEL, 0, 0);
					InvalidateRect(GetDlgItem(hWnd(), IDC_VIEWFRAME), NULL, FALSE);
					return true;
				}
				return false;

			case IDC_FRAMELIST:
				if (HIWORD(wParam) == LBN_SELCHANGE) {
					InvalidateRect(GetDlgItem(hWnd(), IDC_VIEWFRAME), NULL, FALSE);
					ExecuteMessageLoop();
					return true;
				}
				return false;

			case IDC_FULLFRAMES:
				if (HIWORD(wParam) == BN_CLICKED) {
					if (m_HalfFrames) {
						m_HalfFrames = false;
						SendDlgItemMessage(hWnd(), IDC_FULLFRAMES, BM_SETCHECK, BST_CHECKED, 0);
						SendDlgItemMessage(hWnd(), IDC_HALFFRAMES, BM_SETCHECK, BST_UNCHECKED, 0);
						int pos = (int) SendDlgItemMessage(hWnd(), IDC_FRAMELIST, LB_GETCURSEL, 0, 0) / 2;
						SendDlgItemMessage(hWnd(), IDC_FRAMELIST, LB_RESETCONTENT, 0, 0);
						for (int i = 0; i < (m_LastFrame - m_FirstFrame + 1); i++) {
							SendDlgItemMessage(hWnd(), IDC_FRAMELIST, LB_ADDSTRING, 0, i);
						}
						SendDlgItemMessage(hWnd(), IDC_FRAMELIST, LB_SETCURSEL, pos, 0);
						InvalidateRect(GetDlgItem(hWnd(), IDC_VIEWFRAME), NULL, FALSE);
					}
					return true;
				}
				return false;

			case IDC_HALFFRAMES:
				if (HIWORD(wParam) == BN_CLICKED) {
					if (! m_HalfFrames) {
						m_HalfFrames = true;
						SendDlgItemMessage(hWnd(), IDC_FULLFRAMES, BM_SETCHECK, BST_UNCHECKED, 0);
						SendDlgItemMessage(hWnd(), IDC_HALFFRAMES, BM_SETCHECK, BST_CHECKED, 0);
						int pos = (int) SendDlgItemMessage(hWnd(), IDC_FRAMELIST, LB_GETCURSEL, 0, 0) * 2;
						SendDlgItemMessage(hWnd(), IDC_FRAMELIST, LB_RESETCONTENT, 0, 0);
						for (int i = 0; i < (m_LastFrame - m_FirstFrame + 1) * 2; i++) {
							SendDlgItemMessage(hWnd(), IDC_FRAMELIST, LB_ADDSTRING, 0, i);
						}
						SendDlgItemMessage(hWnd(), IDC_FRAMELIST, LB_SETCURSEL, pos, 0);
						InvalidateRect(GetDlgItem(hWnd(), IDC_VIEWFRAME), NULL, FALSE);
					}
					return true;
				}
				return false;

			case IDC_SAVEFRAME:
				if (HIWORD(wParam) == BN_CLICKED) {
					OPENFILENAME ofn;
					TCHAR fileName[MAX_PATH] = TEXT("");
					ZeroMemory(&ofn, sizeof(OPENFILENAME));
					ofn.lStructSize     = sizeof(OPENFILENAME);
					ofn.hwndOwner       = hWnd();
					ofn.hInstance       = hInstance;
					ofn.lpstrFilter     = TEXT("Bitmap files\0*.bmp\0All files\0*.*\0\0");
					ofn.lpstrFile       = fileName;
					ofn.nMaxFile        = MAX_PATH;
					ofn.lpstrInitialDir = NULL;
					ofn.lpstrTitle      = TEXT("Save frame as bitmap");
					ofn.Flags           = OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
					ofn.lpstrDefExt     = TEXT("bmp");
					if (GetSaveFileName(&ofn)) {
						String fileToSave = fileName;
						int index = (int) SendDlgItemMessage(hWnd(), IDC_FRAMELIST, LB_GETCURSEL, 0, 0);
						HBITMAP bitmap = m_FullBitmap[index / (m_HalfFrames ? 2 : 1)];
						BMP image;
						HDC dc = GetDC(hWnd());
						HBITMAPtoBMP(dc, bitmap, image);
						ReleaseDC(hWnd(), dc);
						if (! image.WriteToFile(fileToSave.toAnsi().c_str())) {
							MessageBox(hWnd(), (S("Could not save bitmap file \"") + fileToSave + S("\"")).c_str(),
								APPLICATION, MB_OK | MB_ICONSTOP);
						}
					} else {
						return false;
					}

				}
				return false;

			case IDC_BACK:
				if (HIWORD(wParam) == BN_CLICKED) {
					SendMessage(hWnd(), WM_CLOSE, 0, 0);
					return true;
				}
				return false;

			default:
				return false;
			}
		}

		case WM_MEASUREITEM:
		{
			const AVSWrapper::Info & info = m_AVS.GetInfo();
			LPMEASUREITEMSTRUCT mis = (LPMEASUREITEMSTRUCT) lParam;
			mis->itemHeight = (m_HalfFrames ? 29 : 54);
			mis->itemWidth  = 50 * info.width / info.height;
			return true;
		}

		case WM_DRAWITEM:
		{
			const AVSWrapper::Info & info = m_AVS.GetInfo();
			LPDRAWITEMSTRUCT dis = (LPDRAWITEMSTRUCT) lParam;
			if (0 <= dis->itemID) {
				RECT rect;
				rect.left = rect.top = 0;
				rect.right = dis->rcItem.right - dis->rcItem.left;
				rect.bottom = dis->rcItem.bottom - dis->rcItem.top;
				HBITMAP bitmap = CreateCompatibleBitmap(dis->hDC, rect.right, rect.bottom);
				HDC dc = CreateCompatibleDC(dis->hDC);
				SetStretchBltMode(dc, HALFTONE);
				SelectObject(dc, bitmap);
				FillRect(dc, & rect, ((dis->itemState & ODS_SELECTED) ? GetSysColorBrush(COLOR_HIGHLIGHT) : GetSysColorBrush(COLOR_WINDOW)));
				int width = (rect.bottom - 4) * info.width / info.height * (m_HalfFrames ? 2 : 1);
				int x = (rect.right - width) / 2;
				HDC imageDC = CreateCompatibleDC(dis->hDC);
				SelectObject(imageDC, m_FullBitmap[dis->itemID / (m_HalfFrames ? 2 : 1)]);
				StretchBlt(dc, x, 2, width, rect.bottom - 4, imageDC, 0, 0, info.width,
					info.height / (m_HalfFrames ? 2 : 1), SRCCOPY);
				DeleteDC(imageDC);
				if (dis->itemState & ODS_FOCUS) {
					DrawFocusRect(dc, & rect);
				}
				BitBlt(dis->hDC, dis->rcItem.left, dis->rcItem.top, rect.right, rect.bottom, dc, 0, 0, SRCCOPY);
				DeleteDC(dc);
				DeleteObject(bitmap);
			}
			return true;
		}

		case WM_CLOSE:
		{
			EndDialog(hWnd(), 0);
			return true;
		}
	}
	return false;
}

LRESULT LookAtVideoDialog::ViewFrameProc(HWND vfWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	if (message == WM_PAINT) {
		PAINTSTRUCT ps;
		BeginPaint(vfWnd, & ps);
		RECT rect;
		GetClientRect(vfWnd, & rect);
		HDC dc = GetDC(vfWnd);
		DrawEdge(dc, & rect, EDGE_SUNKEN, BF_RECT | BF_ADJUST);
		int rectWidth  = rect.right  - rect.left;
		int rectHeight = rect.bottom - rect.top;
		int imageWidth  = m_AVS.GetInfo().width;
		int imageHeight = m_AVS.GetInfo().height / (m_HalfFrames ? 2 : 1);
		if (imageWidth  > rectWidth)  imageWidth  = rectWidth;
		if (imageHeight > rectHeight) imageHeight = rectHeight;
		int index = (int) SendDlgItemMessage(hWnd(), IDC_FRAMELIST, LB_GETCURSEL, 0, 0);
		if (index < 0) {
			imageWidth  = 0;
			imageHeight = 0;
		} else {
			HDC imageDC = CreateCompatibleDC(dc);
			if (m_HalfFrames) {
				SelectObject(imageDC, m_FullBitmap[index / 2]);
				for (int line = 0; line < imageHeight; line++) {
					BitBlt(dc, rect.left, rect.top + line, imageWidth, 1, imageDC, 0, line * 2 + index % 2, SRCCOPY);
				}
			} else {
				SelectObject(imageDC, m_FullBitmap[index]);
				BitBlt(dc, rect.left, rect.top, imageWidth, imageHeight, imageDC, 0, 0, SRCCOPY);
			}
			DeleteDC(imageDC);
		}
		HGDIOBJ origBrush = SelectObject(dc, GetStockObject(BLACK_BRUSH));
		HGDIOBJ origPen = SelectObject(dc, GetStockObject(NULL_PEN));
		Rectangle(dc, rect.left, rect.top + imageHeight, rect.left + imageWidth + 1, rect.bottom + 1);
		Rectangle(dc, rect.left + imageWidth, rect.top, rect.right + 1, rect.bottom + 1);
		SelectObject(dc, origPen);
		SelectObject(dc, origBrush);
		ReleaseDC(vfWnd, dc);
		EndPaint(vfWnd, &ps);
		return 0;
	}
	return DefWindowProc(vfWnd, message, wParam, lParam);
}

LRESULT CALLBACK ViewFrameWindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	LookAtVideoDialog * dialog = (LookAtVideoDialog *) GetWindowLong(hWnd, GWL_USERDATA);
	if (dialog) {
		return dialog->ViewFrameProc(hWnd, message, wParam, lParam);
	} else {
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
}

VOID RegisterLookAtVideoDialogControls()
{
	WNDCLASS wcl;
	wcl.style         = 0;
	wcl.cbClsExtra    = 0;
	wcl.cbWndExtra    = 0;
	wcl.hInstance     = hInstance;
	wcl.hIcon         = NULL;
	wcl.hCursor       = LoadCursor(NULL, IDC_ARROW);
	wcl.hbrBackground = (HBRUSH) GetStockObject(HOLLOW_BRUSH);
	wcl.lpszMenuName  = NULL;
	wcl.lpfnWndProc   = ViewFrameWindowProc;
	wcl.lpszClassName = TEXT("ViewFrame");
	RegisterClass(& wcl);
}