#ifndef MANAGEDPTR_H_
#define MANAGEDPTR_H_

template <class Class>
class ManagedPtr 
{
private:
	Class * m_Ptr;
	
public:
	ManagedPtr();
	ManagedPtr(Class * c);
	ManagedPtr(const ManagedPtr<Class> & mp);
	virtual ~ManagedPtr();
	ManagedPtr & operator = (const ManagedPtr<Class> & mp);
	ManagedPtr & operator = (Class * c);
	operator Class * ();
	operator const Class * () const;
	Class * operator()();
	const Class * operator()() const;
	Class * operator->();
	const Class * operator->() const;
	Class & operator*();
	const Class & operator*() const;
	bool operator<(const ManagedPtr<Class> & mp) const;
};

template <class Class>
ManagedPtr<Class>::ManagedPtr()
{
	m_Ptr = 0;
};

template <class Class>
ManagedPtr<Class>::ManagedPtr(const ManagedPtr<Class> & mp)
{
	m_Ptr = 0;
	*this = mp.m_Ptr;
};
	
template <class Class>
ManagedPtr<Class>::ManagedPtr(Class * c)
{
	m_Ptr = 0;
	*this = c;
};
	
template <class Class>
ManagedPtr<Class> & ManagedPtr<Class>::operator = (Class * c)
{
	if (m_Ptr != c) {
		if (m_Ptr) m_Ptr->DelRef();
		m_Ptr = c;
		if (m_Ptr) m_Ptr->AddRef();
	}
	return *this;
};

template <class Class>
ManagedPtr<Class> & ManagedPtr<Class>::operator = (const ManagedPtr<Class> & mp)
{
	return (*this = mp.m_Ptr);
};

template <class Class>
ManagedPtr<Class>::~ManagedPtr()
{
	if (m_Ptr) m_Ptr->DelRef();
};

template <class Class>
ManagedPtr<Class>::operator Class * () 
{
	return m_Ptr;
};

template <class Class>
ManagedPtr<Class>::operator const Class * () const
{
	return m_Ptr;
};

template <class Class>
Class * ManagedPtr<Class>::operator () () 
{
	return m_Ptr;
};

template <class Class>
const Class * ManagedPtr<Class>::operator () () const
{
	return m_Ptr;
};

template <class Class>
Class * ManagedPtr<Class>::operator ->() 
{
	return m_Ptr;
};

template <class Class>
const Class * ManagedPtr<Class>::operator ->() const
{
	return m_Ptr;
};

template <class Class>
Class & ManagedPtr<Class>::operator * () 
{
	return *m_Ptr;
};

template <class Class>
const Class & ManagedPtr<Class>::operator * () const
{
	return *m_Ptr;
};

template <class Class>
bool ManagedPtr<Class>::operator < (const ManagedPtr<Class> &mp) const
{
	return (*m_Ptr) < (*(mp.m_Ptr));
};

#endif // MANAGEDPTR_H_
