#include "menuitem.h"
#include "menu.h"
#include "group.h"
#include "project.h"
#include "config.h"

#include "winutils/winutils.h"

MenuItem::MenuItem(Menu * menu, int selector1, int selector2)
{
	m_Menu       = menu;
	m_IsSelected = false;
	m_Selector1  = selector1;
	m_Selector2  = selector2;
	m_X          = 0;
	m_Y          = 0;
	m_NormalText = g_Config->GetNormalText();
	m_HiliteText = g_Config->GetHiliteText();
	m_FontFace   = g_Config->GetMenuFontFace();
	m_FontSize   = g_Config->GetMenuFontSize();
	m_Bold       = g_Config->GetMenuBold();
	m_Italic     = g_Config->GetMenuItalic();
	m_NormalRect = g_Config->GetNormalRect();
	m_HiliteRect = g_Config->GetHiliteRect();
	m_Right      = 0;
	m_Bottom     = 0;
	m_Thickness  = g_Config->GetThickness();
	m_HRadius    = g_Config->GetHRadius();
	m_VRadius    = g_Config->GetVRadius();
	m_FillNormal = g_Config->GetFillNormal();
	m_FillHilite = g_Config->GetFillHilite();
	m_AutoResize = g_Config->GetAutoResize();
	m_HMargin    = g_Config->GetHMargin();
	m_VMargin    = g_Config->GetVMargin();
	m_Alignment  = g_Config->GetAlignment();
	m_IsValid    = false;
	m_Index      = -1;
}

MenuItem::~MenuItem()
{
}

Menu * MenuItem::GetMenu() const
{
	return m_Menu;
}

bool MenuItem::IsSelected() const
{
	return m_IsSelected;
}

void MenuItem::SetSelected(bool selected)
{
	m_IsSelected = selected;
}

int MenuItem::GetSelector1() const
{
	return m_Selector1;
}

bool MenuItem::SetSelector1(int selector1, bool setChanged)
{
	if (m_Selector1 != selector1) {
		m_Selector1 = selector1;
		if (setChanged) SetChanged();
	}
	return true;
}

int MenuItem::GetSelector2() const
{
	return m_Selector2;
}

bool MenuItem::SetSelector2(int selector2, bool setChanged)
{
	if (m_Selector2 != selector2) {
		m_Selector2 = selector2;
		if (setChanged) SetChanged();
	}
	return true;
}

int MenuItem::GetX() const
{
	return m_X;
}

bool MenuItem::SetX(int x, bool setChanged)
{
	if (x < 0) return false;
	if (x >= GetMenu()->GetGroup()->GetProject()->GetVideoWidth()) {
		x = GetMenu()->GetGroup()->GetProject()->GetVideoWidth() - 1;
	}
	if (m_X != x) {
		m_X = x;
		if (m_Right < m_X) {
			m_Right = m_X;
		}
		if (setChanged) SetChanged();
	}
	return true;
}

int MenuItem::GetY() const
{
	return m_Y;
}

bool MenuItem::SetY(int y, bool setChanged)
{
	if (y < 0) return false;
	if (y >= GetMenu()->GetGroup()->GetProject()->GetVideoHeight()) {
		y = GetMenu()->GetGroup()->GetProject()->GetVideoHeight() - 1;
	}
	if (m_Y != y) {
		m_Y = y;
		if (m_Bottom < m_Y) {
			m_Bottom = m_Y;
		}
		if (setChanged) SetChanged();
	}
	return true;
}

bool MenuItem::GetNormalText() const
{
	return m_NormalText;
}

bool MenuItem::SetNormalText(bool normalText, bool setChanged)
{
	if (m_NormalText != normalText) {
		m_NormalText = normalText;
		if (setChanged) SetChanged();
	}
	return true;
}

bool MenuItem::GetHiliteText() const
{
	return m_HiliteText;
}

bool MenuItem::SetHiliteText(bool hiliteText, bool setChanged)
{
	if (m_HiliteText != hiliteText) {
		m_HiliteText = hiliteText;
		if (setChanged) SetChanged();
	}
	return true;
}

const String & MenuItem::GetCaption() const
{
	return m_Caption;
}

bool MenuItem::SetCaption(const String & caption, bool setChanged)
{
	if (m_Caption != caption) {
		m_Caption = caption;
		if (setChanged) SetChanged();
	}
	return true;
}

const String & MenuItem::GetFontFace()
{
	return m_FontFace;
}

bool MenuItem::SetFontFace(const String & fontFace, bool setChanged)
{
	if (m_FontFace != fontFace) {
		m_FontFace = fontFace;
		if (setChanged) SetChanged();
	}
	return true;
}

int MenuItem::GetFontSize() const
{
	return m_FontSize;
}

bool MenuItem::SetFontSize(int fontSize, bool setChanged)
{
	if (0 <= fontSize) {
		if (m_FontSize != fontSize) {
			m_FontSize = fontSize;
			if (setChanged) SetChanged();
		}
		return true;
	}
	return false;
}

bool MenuItem::GetBold() const
{
	return m_Bold;
}

bool MenuItem::SetBold(bool bold, bool setChanged)
{
	if (m_Bold != bold) {
		m_Bold = bold;
		if (setChanged) SetChanged();
	}
	return true;
}

bool MenuItem::GetItalic() const
{
	return m_Italic;
}

bool MenuItem::SetItalic(bool italic, bool setChanged)
{
	if (m_Italic != italic) {
		m_Italic = italic;
		if (setChanged) SetChanged();
	}
	return true;
}

bool MenuItem::GetRectangle() const
{
	return (m_NormalRect || m_HiliteRect);
}

bool MenuItem::GetNormalRect() const
{
	return m_NormalRect;
}

bool MenuItem::SetNormalRect(bool normalRect, bool setChanged)
{
	if (m_NormalRect != normalRect) {
		m_NormalRect = normalRect;
		if (setChanged) SetChanged();
	}
	return true;
}

bool MenuItem::GetHiliteRect() const
{
	return m_HiliteRect;
}

bool MenuItem::SetHiliteRect(bool hiliteRect, bool setChanged)
{
	if (m_HiliteRect != hiliteRect) {
		m_HiliteRect = hiliteRect;
		if (setChanged) SetChanged();
	}
	return true;
}

int MenuItem::GetRight() const
{
	return m_Right;
}

bool MenuItem::SetRight(int right, bool setChanged)
{
	if (right < 0) return false;
	if (right >= GetMenu()->GetGroup()->GetProject()->GetVideoWidth()) {
		right = GetMenu()->GetGroup()->GetProject()->GetVideoWidth() - 1;
	}
	if (right < m_X) {
		right = m_X;
	}
	if (m_Right != right) {
		m_Right = right;
		if (setChanged) SetChanged();
	}
	return true;
}

int MenuItem::GetBottom() const
{
	return m_Bottom;
}

bool MenuItem::SetBottom(int bottom, bool setChanged)
{
	if (bottom < 0) return false;
	if (bottom >= GetMenu()->GetGroup()->GetProject()->GetVideoHeight()) {
		bottom = GetMenu()->GetGroup()->GetProject()->GetVideoHeight() - 1;
	}
	if (bottom < m_Y) {
		bottom = m_Y;
	}
	if (m_Bottom != bottom) {
		m_Bottom = bottom;
		if (setChanged) SetChanged();
	}
	return true;
}

int MenuItem::GetThickness() const
{
	return m_Thickness;
}

bool MenuItem::SetThickness(int thickness, bool setChanged)
{
	if (0 < thickness) {
		if (m_Thickness != thickness) {
			m_Thickness = thickness;
			if (setChanged) SetChanged();
		}
		return true;
	}
	return false;
}

int MenuItem::GetHRadius() const
{
	return m_HRadius;
}

bool MenuItem::SetHRadius(int hRadius, bool setChanged)
{
	if (0 <= hRadius) {
		if (m_HRadius != hRadius) {
			m_HRadius = hRadius;
			if (! m_VRadius) m_VRadius = hRadius;
			if (setChanged) SetChanged();
		}
		return true;
	}
	return false;
}

int MenuItem::GetVRadius() const
{
	return m_VRadius;
}

bool MenuItem::SetVRadius(int vRadius, bool setChanged)
{
	if (0 <= vRadius) {
		if (m_VRadius != vRadius) {
			m_VRadius = vRadius;
			if (! m_HRadius) m_HRadius = vRadius;
			if (setChanged) SetChanged();
		}
		return true;
	}
	return false;
}

bool MenuItem::GetFillNormal() const
{
	return m_FillNormal;
}

bool MenuItem::SetFillNormal(bool fillNormal, bool setChanged)
{
	if (m_FillNormal != fillNormal) {
		m_FillNormal = fillNormal;
		if (setChanged) SetChanged();
	}
	return true;
}

bool MenuItem::GetFillHilite() const
{
	return m_FillHilite;
}

bool MenuItem::SetFillHilite(bool fillHilite, bool setChanged)
{
	if (m_FillHilite != fillHilite) {
		m_FillHilite = fillHilite;
		if (setChanged) SetChanged();
	}
	return true;
}

bool MenuItem::GetAutoResize() const
{
	return m_AutoResize;
}

bool MenuItem::SetAutoResize(bool autoResize, bool setChanged)
{
	if (m_AutoResize != autoResize) {
		m_AutoResize = autoResize;
		if (setChanged) SetChanged();
	}
	return true;
}

int MenuItem::GetHMargin() const
{
	return m_HMargin;
}

bool MenuItem::SetHMargin(int hMargin, bool setChanged)
{
	if (0 <= hMargin) {
		if (m_HMargin != hMargin) {
			m_HMargin = hMargin;
			if (setChanged) SetChanged();
		}
		return true;
	}
	return false;
}

int MenuItem::GetVMargin() const
{
	return m_VMargin;
}

bool MenuItem::SetVMargin(int vMargin, bool setChanged)
{
	if (0 <= vMargin) {
		if (m_VMargin != vMargin) {
			m_VMargin = vMargin;
			if (setChanged) SetChanged();
		}
		return true;
	}
	return false;
}

int MenuItem::GetAlignment() const
{
	return m_Alignment;
}

bool MenuItem::SetAlignment(int alignment, bool setChanged)
{
	if (0 <= alignment && alignment < ALIGN_MAX) {
		if (m_Alignment != alignment) {
			m_Alignment = alignment;
			if (setChanged) SetChanged();
		}
		return true;
	}
	return false;
}

void MenuItem::SetChanged()
{
	m_Menu->SetChanged();
}

int MenuItem::GetIndex() const
{
	return m_Index;
}

void MenuItem::SetIndex(int index)
{
	m_Index = index;
}

String MenuItem::GetIdent()
{
	switch (GetMenu()->GetType()) {
	case MENU_ROOT:
		return S("Entry ") + String(GetIndex() + 1) 
			+ S(" of root menu of group ") + String(GetMenu()->GetGroup()->GetIndex() + 1) 
			+ S(" (") + GetShortDescr() + S("): ");
	case MENU_COMBINED:
		return S("Entry ") + String(GetIndex() + 1) 
			+ S(" of audio + subtitle menu of group ") + String(GetMenu()->GetGroup()->GetIndex() + 1) 
			+ S(" (") + GetShortDescr() + S("): ");
	case MENU_AUDIO:
		return S("Entry ") + String(GetIndex() + 1) 
			+ S(" of audio menu of group ") + String(GetMenu()->GetGroup()->GetIndex() + 1) 
			+ S(" (") + GetShortDescr() + S("): ");
	case MENU_SUBTITLE:
		return S("Entry ") + String(GetIndex() + 1) 
			+ S(" of subtitle menu of group ") + String(GetMenu()->GetGroup()->GetIndex() + 1) 
			+ S(" (") + GetShortDescr() + S("): ");
	default:
		return S("");
	}
}

String MenuItem::Recalculate()
{
	String error;

	switch (GetMenu()->GetType()) 
	{
	case MENU_ROOT:
		switch (GetSelector1()) 
		{
		case ROOTMENU_PLAYTITLE:
			if (GetSelector2() >= GetMenu()->GetGroup()->GetTitleCount()) {
				error += (GetIdent() + S("Invalid because there are only ") + String(GetMenu()->GetGroup()->GetTitleCount()) + S(" titles.\n"));
			}
			break;
		case ROOTMENU_GOTOCOMBINED:
			if (GetMenu()->GetGroup()->GetMenuType() != MENUTYPE_COMBINED) {
				error += (GetIdent() + S("Invalid because there's no audio + subtitle menu.\n"));
			}
			break;
		case ROOTMENU_GOTOAUDIO:
			if (GetMenu()->GetGroup()->GetMenuType() != MENUTYPE_SEPARATE || ! GetMenu()->GetGroup()->HasAudioMenu()) {
				error += (GetIdent() + S("Invalid because there's no audio menu.\n"));
			}
			break;
		case ROOTMENU_GOTOSUBTITLE:
			if (GetMenu()->GetGroup()->GetMenuType() != MENUTYPE_SEPARATE || ! GetMenu()->GetGroup()->HasSubtitleMenu()) {
				error += (GetIdent() + S("Invalid, because there's no subtitle menu.\n"));
			}
			break;
		case ROOTMENU_GOTOTITLESET:
			if (GetSelector2() >= GetMenu()->GetGroup()->GetProject()->GetGroupCount()) {
				error += (GetIdent() + S("Invalid because there are only ") + String(GetMenu()->GetGroup()->GetProject()->GetGroupCount()) + S(" groups.\n"));
			}
			break;
		case ROOTMENU_RESUME:
			if (! GetMenu()->GetGroup()->GetTitleCount()) {
				error += (GetIdent() + S("Invalid because there no titles in the group.\n"));
			}
			break;
		}
		break;
	case MENU_COMBINED:
	case MENU_AUDIO:
	case MENU_SUBTITLE:
		if (GetSelector1() >= GetMenu()->GetGroup()->GetAudioCount()) {
			error += (GetIdent() + S("Invalid because there are only ") + String(GetMenu()->GetGroup()->GetAudioCount()) + S(" audio tracks.\n"));
		}
		if (GetSelector2() > GetMenu()->GetGroup()->GetSubtitleCount()) {
			error += (GetIdent() + S("Invalid because there are only ") + String(GetMenu()->GetGroup()->GetSubtitleCount()) + S(" subtitle tracks.\n"));
		}
		break;
	}
	
	HFONT font = NULL;
	if (m_Caption.length()) {
		font = CreateFont();
		m_IsValid = (font != NULL);
		if (! m_IsValid) {
			error += (GetIdent() + S("There's no such font as ") + GetFontFace().enclose() + S("\n"));
		}
	} else {
		m_IsValid = true;
	}

	SIZE size;
	size.cx = 0;
	size.cy = 0;
	if (m_Caption.length()) {
		HDC dc = CreateCompatibleDC(NULL);
		SelectObject(dc, font);
		GetTextExtentPoint32(dc, m_Caption.c_str(), (int) m_Caption.length(), & size);
		DeleteDC(dc);
		DeleteObject(font);
		if (m_AutoResize) {
			m_Right  = m_X + size.cx + 2 * m_HMargin;
			m_Bottom = m_Y + size.cy + 2 * m_VMargin;
		}
	}
	m_BaseRect.left = m_X;
	m_BaseRect.top  = m_Y;
	m_BaseRect.right  = m_Right;
	m_BaseRect.bottom = m_Bottom;
	switch (m_Alignment) {
	case ALIGN_TOPLEFT:
		m_TextRect.left = m_BaseRect.left + m_HMargin;
		m_TextRect.top  = m_BaseRect.top  + m_VMargin;
		break;
	case ALIGN_TOPCENTER:
		m_TextRect.left = m_BaseRect.left + (m_BaseRect.right - m_BaseRect.left - size.cx) / 2;
		m_TextRect.top  = m_BaseRect.top  + m_VMargin;
		break;
	case ALIGN_TOPRIGHT:
		m_TextRect.left = m_BaseRect.right - size.cx - m_HMargin;
		m_TextRect.top  = m_BaseRect.top  + m_VMargin;
		break;
	case ALIGN_CENTERLEFT:
		m_TextRect.left = m_BaseRect.left + m_HMargin;
		m_TextRect.top  = m_BaseRect.top  + (m_BaseRect.bottom - m_BaseRect.top - size.cy) / 2;
		break;
	case ALIGN_CENTERCENTER:
		m_TextRect.left = m_BaseRect.left + (m_BaseRect.right - m_BaseRect.left - size.cx) / 2;
		m_TextRect.top  = m_BaseRect.top  + (m_BaseRect.bottom - m_BaseRect.top - size.cy) / 2;
		break;
	case ALIGN_CENTERRIGHT:
		m_TextRect.left = m_BaseRect.right - size.cx - m_HMargin;
		m_TextRect.top  = m_BaseRect.top  + (m_BaseRect.bottom - m_BaseRect.top - size.cy) / 2;
		break;
	case ALIGN_BOTTOMLEFT:
		m_TextRect.left = m_BaseRect.left + m_HMargin;
		m_TextRect.top  = m_BaseRect.bottom - size.cy - m_VMargin;
		break;
	case ALIGN_BOTTOMCENTER:
		m_TextRect.left = m_BaseRect.left + (m_BaseRect.right - m_BaseRect.left - size.cx) / 2;
		m_TextRect.top  = m_BaseRect.bottom - size.cy - m_VMargin;
		break;
	case ALIGN_BOTTOMRIGHT:
		m_TextRect.left = m_BaseRect.right - size.cx - m_HMargin;
		m_TextRect.top  = m_BaseRect.bottom - size.cy - m_VMargin;
		break;
	}
	m_TextRect.right = m_TextRect.left + size.cx;
	m_TextRect.bottom = m_TextRect.top + size.cy;
	m_DrawnRect = m_BaseRect;
	if (GetRectangle()) {
		double aspectCorr = GetMenu()->GetGroup()->GetProject()->GetAspectCorr(GetMenu()->GetGroup()->GetMenuAspectRatio());
		m_DrawnRect.left   -= (int) (0.5 + m_Thickness * aspectCorr);
		m_DrawnRect.top    -= m_Thickness;
		m_DrawnRect.right  += (int) (0.5 + m_Thickness * aspectCorr);
		m_DrawnRect.bottom += m_Thickness;
	}
	if (m_DrawnRect.left   > m_TextRect.left)   m_DrawnRect.left   = m_TextRect.left;
	if (m_DrawnRect.top    > m_TextRect.top)    m_DrawnRect.top    = m_TextRect.top;
	if (m_DrawnRect.right  < m_TextRect.right)  m_DrawnRect.right  = m_TextRect.right;
	if (m_DrawnRect.bottom < m_TextRect.bottom) m_DrawnRect.bottom = m_TextRect.bottom;
	m_AdjustedRect.left = (m_DrawnRect.left < 0 ? 0 : m_DrawnRect.left);
	m_AdjustedRect.top  = (m_DrawnRect.top  < 0 ? 0 : m_DrawnRect.top);
	m_AdjustedRect.right = (m_DrawnRect.right <= GetMenu()->GetGroup()->GetProject()->GetVideoWidth() ?
							m_DrawnRect.right : GetMenu()->GetGroup()->GetProject()->GetVideoWidth() - 1);
	m_AdjustedRect.bottom = (m_DrawnRect.bottom <= GetMenu()->GetGroup()->GetProject()->GetVideoHeight() ?
							 m_DrawnRect.bottom : GetMenu()->GetGroup()->GetProject()->GetVideoHeight() - 1);
	if (m_AdjustedRect.left > m_AdjustedRect.right || m_AdjustedRect.top > m_AdjustedRect.bottom) {
		m_IsValid = false;
		error += (GetIdent() + S("Menu entry rectangle is outside the video image.\n"));
	} 
	#define RSIZE 6
	m_DragRect[DRAG_TOPLEFT].left   = m_BaseRect.left - RSIZE;
	m_DragRect[DRAG_TOPLEFT].top    = m_BaseRect.top  - RSIZE;
	m_DragRect[DRAG_TOPLEFT].right  = m_BaseRect.left;
	m_DragRect[DRAG_TOPLEFT].bottom = m_BaseRect.top;
	m_DragRect[DRAG_TOP].left   = m_BaseRect.left + (m_BaseRect.right - m_BaseRect.left - RSIZE) / 2;
	m_DragRect[DRAG_TOP].top    = m_BaseRect.top  - RSIZE;
	m_DragRect[DRAG_TOP].right  = m_DragRect[DRAG_TOP].left + RSIZE;
	m_DragRect[DRAG_TOP].bottom = m_BaseRect.top;
	m_DragRect[DRAG_TOPRIGHT].left   = m_BaseRect.right;
	m_DragRect[DRAG_TOPRIGHT].top    = m_BaseRect.top  - RSIZE;
	m_DragRect[DRAG_TOPRIGHT].right  = m_BaseRect.right + RSIZE;
	m_DragRect[DRAG_TOPRIGHT].bottom = m_BaseRect.top;
	m_DragRect[DRAG_LEFT].left   = m_BaseRect.left - RSIZE;
	m_DragRect[DRAG_LEFT].top    = m_BaseRect.top + (m_BaseRect.bottom - m_BaseRect.top - RSIZE) / 2;
	m_DragRect[DRAG_LEFT].right  = m_BaseRect.left;
	m_DragRect[DRAG_LEFT].bottom = m_DragRect[DRAG_LEFT].top + RSIZE;
	m_DragRect[DRAG_RIGHT].left   = m_BaseRect.right;
	m_DragRect[DRAG_RIGHT].top    = m_BaseRect.top + (m_BaseRect.bottom - m_BaseRect.top - RSIZE) / 2;
	m_DragRect[DRAG_RIGHT].right  = m_BaseRect.right + RSIZE;
	m_DragRect[DRAG_RIGHT].bottom = m_DragRect[DRAG_RIGHT].top + RSIZE;
	m_DragRect[DRAG_BOTTOMLEFT].left   = m_BaseRect.left - RSIZE;
	m_DragRect[DRAG_BOTTOMLEFT].top    = m_BaseRect.bottom;
	m_DragRect[DRAG_BOTTOMLEFT].right  = m_BaseRect.left;
	m_DragRect[DRAG_BOTTOMLEFT].bottom = m_BaseRect.bottom + RSIZE;
	m_DragRect[DRAG_BOTTOM].left   = m_BaseRect.left + (m_BaseRect.right - m_BaseRect.left - RSIZE) / 2;
	m_DragRect[DRAG_BOTTOM].top    = m_BaseRect.bottom;
	m_DragRect[DRAG_BOTTOM].right  = m_DragRect[DRAG_BOTTOM].left + RSIZE;
	m_DragRect[DRAG_BOTTOM].bottom = m_BaseRect.bottom + RSIZE;
	m_DragRect[DRAG_BOTTOMRIGHT].left   = m_BaseRect.right;
	m_DragRect[DRAG_BOTTOMRIGHT].top    = m_BaseRect.bottom;
	m_DragRect[DRAG_BOTTOMRIGHT].right  = m_BaseRect.right + RSIZE;
	m_DragRect[DRAG_BOTTOMRIGHT].bottom = m_BaseRect.bottom + RSIZE;

	for (int mindex = GetIndex() - 1; mindex >= 0; mindex--) {
		const RECT & r1 = GetAdjustedRect();
		const RECT & r2 = GetMenu()->GetMenuItem(mindex)->GetAdjustedRect();
		bool intersecting = !
			((r1.right  <= r2.left)  ||
			 (r1.bottom <= r2.top)   ||
			 (r1.left   >= r2.right) ||
			 (r1.top    >= r2.bottom));
		if (intersecting) {
			error += (GetIdent() + S("Intersects with entry ") + String(mindex + 1) 
				+ S(" (") + GetMenu()->GetMenuItem(mindex)->GetShortDescr() + S(")") + S(".\n"));
		}
	}

	if (font) {
		DeleteObject(font);
	}

	return error;
}

bool MenuItem::IsValid() const
{
	return m_IsValid;
}

const RECT & MenuItem::GetBaseRect() const
{
	return m_BaseRect;
}

const RECT & MenuItem::GetTextRect() const
{
	return m_TextRect;
}

const RECT & MenuItem::GetDrawnRect() const
{
	return m_DrawnRect;
}

const RECT & MenuItem::GetAdjustedRect() const
{
	return m_AdjustedRect;
}

const RECT & MenuItem::GetDragRect(int index) const
{
	return m_DragRect[index];
}

HFONT MenuItem::CreateFont() const
{
	LOGFONT lf;
	if (GetFontInfo(m_FontFace.c_str(), & lf, NULL)) {
		double aspectCorr = GetMenu()->GetGroup()->GetProject()->GetAspectCorr(GetMenu()->GetGroup()->GetMenuAspectRatio());
		double fontWHRatio = 1.0 * lf.lfWidth / lf.lfHeight;
		lf.lfHeight    = m_FontSize;
		lf.lfWidth     = (int) (0.5 + 1.0 * lf.lfHeight * fontWHRatio * aspectCorr);
		lf.lfWeight    = (m_Bold ? FW_BOLD : FW_NORMAL);
		lf.lfItalic    = m_Italic;
		return CreateFontIndirect(& lf);
	} 
	return NULL;	
}

void MenuItem::GetDescription(StringVectorVector & strings) const
{
	StringVector line;
	switch (GetMenu()->GetType()) {
	case MENU_ROOT:
		switch (m_Selector1) {
		case ROOTMENU_PLAYTITLE:
			line.push_back(S("Play "));
			line.push_back(S("title ") + String(m_Selector2 + 1));
			break;
		case ROOTMENU_GOTOCOMBINED:
			line.push_back(S("Go to "));
			line.push_back(S("audio & subtitle menu"));
			break;
		case ROOTMENU_GOTOAUDIO:
			line.push_back(S("Go to "));
			line.push_back(S("audio menu"));
			break;
		case ROOTMENU_GOTOSUBTITLE:
			line.push_back(S("Go to "));
			line.push_back(S("subtitle menu"));
			break;
		case ROOTMENU_GOTOTITLESET:
			line.push_back(S("Go to "));
			line.push_back(S("group ") + String(m_Selector2 + 1));
			break;
		case ROOTMENU_RESUME:
			line.push_back(S(""));
			line.push_back(S("Resume play"));
			break;
		}
		break;
	case MENU_COMBINED:
		line.push_back(S("Select "));
		line.push_back(S("audio ") + String(m_Selector1 + 1) + S(" + ") + 
			(m_Selector2 == 0 ? S("no subtitle") : S("subtitle ") + String(m_Selector2)));
		break;
	case MENU_AUDIO:
		line.push_back(S("Select "));
		line.push_back(S("audio ") + String(m_Selector1 + 1));
		break;
	case MENU_SUBTITLE:
		line.push_back(S("Select "));
		line.push_back(m_Selector2 == 0 ? S("no subtitle") : S("subtitle ") + String(m_Selector2));
		break;
	}
	strings.push_back(line);
}

String MenuItem::GetShortDescr() const
{
	StringVectorVector strings;
	GetDescription(strings);
	String descr;
	for (StringVector::iterator si = strings[0].begin(); si != strings[0].end(); si++) {
		descr += *si;
	}
	return descr;
}

bool MenuItem::operator < (const MenuItem & menuItem) const
{
	return (m_Selector1 < menuItem.m_Selector1 ||
		(m_Selector1 == menuItem.m_Selector1 && m_Selector2 < menuItem.m_Selector2));
}

bool MenuItem::Load(csv_file_t csv, int & lineno, int nest)
{
	bool ok = true;

	int field = 1;
	ok = ok && SetSelector1(csv_get_int(csv, lineno, ++field, m_Selector1), false);
	ok = ok && SetSelector2(csv_get_int(csv, lineno, ++field, m_Selector2), false);
	m_IsSelected = (csv_get_int(csv, lineno, ++field, m_IsSelected) != 0);
	++field; 
	ok = ok && SetCaption(String().fromUtf8(csv_get_str(csv, lineno, ++field, m_Caption.toUtf8().c_str())), false);
	ok = ok && SetX(csv_get_int(csv, lineno, ++field, m_X), false);
	ok = ok && SetY(csv_get_int(csv, lineno, ++field, m_Y), false);
	ok = ok && SetAlignment(csv_get_int(csv, lineno, ++field, m_Alignment), false);
	ok = ok && SetFontFace(String().fromUtf8(csv_get_str(csv, lineno, ++field, m_FontFace.toUtf8().c_str())), false);
	ok = ok && SetFontSize(csv_get_int(csv, lineno, ++field, m_FontSize), false);
	ok = ok && SetBold(csv_get_int(csv, lineno, ++field, m_Bold ? 1 : 0) != 0, false);
	ok = ok && SetItalic(csv_get_int(csv, lineno, ++field, m_Italic ? 1 : 0) != 0, false);
	ok = ok && SetHiliteRect(csv_get_int(csv, lineno, ++field, m_HiliteRect ? 1 : 0) != 0, false);
	ok = ok && SetRight(csv_get_int(csv, lineno, ++field, m_Right), false);
	ok = ok && SetBottom(csv_get_int(csv, lineno, ++field, m_Bottom), false);
	ok = ok && SetThickness(csv_get_int(csv, lineno, ++field, m_Thickness), false);
	int distanceFromText = csv_get_int(csv, lineno, ++field, 0);
	ok = ok && SetHMargin(distanceFromText, false);
	ok = ok && SetVMargin(distanceFromText, false);
	++field;
	int edgeRadius = csv_get_int(csv, lineno, ++field, 0);
	ok = ok && SetHRadius(edgeRadius, false);
	ok = ok && SetVRadius(edgeRadius, false);
	ok = ok && SetFillHilite(csv_get_int(csv, lineno, ++field, m_FillHilite ? 1 : 0) != 0, false);
	ok = ok && SetFillNormal(csv_get_int(csv, lineno, ++field, m_FillNormal ? 1 : 0) != 0, false);
	ok = ok && SetNormalText(csv_get_int(csv, lineno, ++field, m_NormalText ? 1 : 0) != 0, false);
	ok = ok && SetHiliteText(csv_get_int(csv, lineno, ++field, m_HiliteText ? 1 : 0) != 0, false);
	ok = ok && SetNormalRect(csv_get_int(csv, lineno, ++field, m_NormalRect ? 1 : 0) != 0, false);
	ok = ok && SetHiliteRect(csv_get_int(csv, lineno, ++field, m_HiliteRect ? 1 : 0) != 0, false);
	ok = ok && SetHRadius(csv_get_int(csv, lineno, ++field, m_HRadius), false);
	ok = ok && SetVRadius(csv_get_int(csv, lineno, ++field, m_VRadius), false);
	ok = ok && SetAutoResize(csv_get_int(csv, lineno, ++field, m_AutoResize ? 1 : 0) != 0, false);
	ok = ok && SetAlignment(csv_get_int(csv, lineno, ++field, m_Alignment), false);

	lineno++;
	while (ok && lineno < csv->count && csv_get_int(csv, lineno, 0, 0) >= nest) {
		lineno++;
	}

	return ok;
}

void MenuItem::Save(FILE * f, int nest) const
{
	fprintf(f, "%d,menuitem,%d,%d,%d,%d,%s,%d,%d,%d,%s,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n", nest,
		m_Selector1, 
		m_Selector2, 
		(m_IsSelected ? 1 : 0),
		(m_Caption.length() ? 1 : 0),
		m_Caption.enclose().toUtf8().c_str(),
		m_X, 
		m_Y, 
		m_Alignment % 3, 
		m_FontFace.enclose().toUtf8().c_str(),
		m_FontSize,
		(m_Bold ? 1 : 0),
		(m_Italic ? 1 : 0),
		(GetRectangle() ? 1 : 0),
		m_Right,
		m_Bottom,
		m_Thickness,
		(m_HMargin > m_VMargin ? m_HMargin : m_VMargin),
		(m_HRadius && m_VRadius ? 1 : 0),
		(m_HRadius > m_VRadius ? m_HRadius : m_VRadius),
		(m_FillHilite ? 1 : 0),
		(m_FillNormal ? 1 : 0),
		(m_NormalText ? 1 : 0),
		(m_HiliteText ? 1 : 0),
		(m_NormalRect ? 1 : 0),
		(m_HiliteRect ? 1 : 0),
		m_HRadius,
		m_VRadius,
		(m_AutoResize ? 1 : 0),
		m_Alignment);
}
