#include "string.h"

#include <windows.h>
#include <stdio.h>
#include <string.h>

String & String::operator=(int x)
{
	wchar_t s[16];
	wsprintfW(s, L"%d", x);
	return (*this = s);
}

String & String::operator=(double x)
{
	char s[32];
	wchar_t ss[32];
	sprintf(s, "%f", x);
	MultiByteToWideChar(CP_ACP, 0, s, -1, ss, 32);
	return (*this = ss);
}

String & String::operator+=(int x)
{
	wchar_t s[16];
	wsprintfW(s, L"%d", x);
	return (*this += s);
}

String & String::operator+=(double x)
{
	char s[32];
	wchar_t ss[32];
	sprintf(s, "%f", x);
	MultiByteToWideChar(CP_ACP, 0, s, -1, ss, 32);
	return (*this += ss);
}


String & String::fromWindow(HWND hWnd)
{
	int length = GetWindowTextLength(hWnd) + 1;
	wchar_t * s = new wchar_t[length];
	GetWindowText(hWnd, s, length);
	*this = s;
	delete [] s;
	return *this;
}

String & String::fromLength(double length)
{
	int hour   = (int) (length / 60 / 60);
	int minute = (int) ((length - hour * 60 * 60) / 60);
	int second = (int) ((length - hour * 60 * 60 - minute * 60));
	int milli  = (int) ((length - hour * 60 * 60 - minute * 60 - second) * 1000 + 0.5);
	wchar_t s[32];
	if (hour) {
		if (milli) {
			wsprintfW(s, L"%d:%02d:%02d.%03d", hour, minute, second, milli);
		} else {
			wsprintfW(s, L"%d:%02d:%02d", hour, minute, second);
		}
	} else {
		if (milli) {
			wsprintfW(s, L"%d:%02d.%03d", minute, second, milli);
		} else {
			wsprintfW(s, L"%d:%02d", minute, second);
		}
	}

	*this = s;
	return *this;
}

String & String::fromUtf8(const std::string & s)
{
	size_t length = s.length() + 1;
	wchar_t * ss = new wchar_t[length];
	MultiByteToWideChar(CP_UTF8, 0, s.c_str(), -1, ss, (int) length);
	*this = ss;
	delete [] ss;
	return *this;
}
	
String & String::fromUtf8(const char * s)
{
	size_t length = strlen(s) + 1;
	wchar_t * ss = new wchar_t[length];
	MultiByteToWideChar(CP_UTF8, 0, s, -1, ss, (int) length);
	*this = ss;
	delete [] ss;
	return *this;
}

String & String::fromAnsi(const std::string & s)
{
	size_t length = s.length() + 1;
	wchar_t * ss = new wchar_t[length];
	MultiByteToWideChar(CP_ACP, 0, s.c_str(), -1, ss, (int) length);
	*this = ss;
	delete [] ss;
	return *this;
}
	
String & String::fromAnsi(const char * s)
{
	size_t length = strlen(s) + 1;
	wchar_t * ss = new wchar_t[length];
	MultiByteToWideChar(CP_ACP, 0, s, -1, ss, (int) length);
	*this = ss;
	delete [] ss;
	return *this;
}

std::string String::toUtf8() const
{
	size_t length = this->length() * 2 + 1;
	char * ss = new char[length];
	WideCharToMultiByte(CP_UTF8, 0, this->c_str(), -1, ss, (int) length, NULL, NULL);
	std::string s = ss;
	delete [] ss;
	return s;
}
std::string String::toAnsi() const
{
	size_t length = this->length() * 2 + 1;
	char * ss = new char[length];
	WideCharToMultiByte(CP_ACP, 0, this->c_str(), -1, ss, (int) length, NULL, NULL);
	std::string s = ss;
	delete [] ss;
	return s;
}

String String::crlf() const
{
	String result;
	for (String::const_iterator si = begin(); si != end(); si++) {
		if (*si == L'\n') result += L'\r';
		result += *si;
	}
	return result;
}

String String::lfOnly() const
{
	String result;
	for (String::const_iterator si = begin(); si != end(); si++) {
		if (*si != L'\r') result += *si;
	}
	return result;
}

String String::trim() const
{
	int begin = 0;
	int end = (int) length();
	while ((begin < end) && (iswspace(operator[](begin)))) begin++;
	while ((begin < end) && (iswspace(operator[](end - 1)))) end--;
	return substr(begin, end - begin);
}

String String::firstLine() const
{
	String s;
	for (const_iterator c = begin(); c != end() && (*c) != L'\r' && (*c) != L'\n'; c++) {
		s += (*c);
	}
	return s;
}

String String::enclose(wchar_t c) const
{
	String result(c);
	for (String::const_iterator si = begin(); si != end(); si++) {
		if (*si == c) result += c;
		result += *si;
	}
	result += c;
	return result;
}

String String::xmlenc() const
{
	String result;
	for (String::const_iterator si = begin(); si != end(); si++) {
		switch (*si) {
		case L'"':
			result += L"&quot;";
			break;
		case L'<': 
			result += L"&lt;";
			break;
		case L'>':
			result += L"&gt;";
			break;
		case L'&':
			result += L"&amp;";
			break;
		default:
			result += *si;
		}
	}
	return result;
}

String String::toShortPath() const
{
	if (! length()) return String();
	int length = (int) GetShortPathName(c_str(), NULL, 0) + 1;
	wchar_t * s = new wchar_t[length];
	GetShortPathName(c_str(), s, length);
	String result = String(s);
	delete [] s;
	return result;
}

String String::toFullPath() const
{
	if (! trim().length()) return String();
	int length = (int) GetFullPathName(c_str(), 0, NULL, NULL) + 1;
	wchar_t * s = new wchar_t[length];
	GetFullPathName(c_str(), length, s, NULL);
	String result = String(s);
	delete [] s;
	return result;
}

String String::toFileName() const
{
	if (! length()) return String();
	int length = (int) GetFullPathName(c_str(), 0, NULL, NULL) + 1;
	wchar_t * s = new wchar_t[length];
	wchar_t * ss;
	GetFullPathName(c_str(), length, s, & ss);
	String result = String(ss);
	delete [] s;
	return result;
}

String String::replaceFileName(const String & fileName) const
{
	if (! length()) return fileName;
	int length = (int) GetFullPathName(c_str(), 0, NULL, NULL) + 1;
	wchar_t * s = new wchar_t[length];
	wchar_t * ss;
	GetFullPathName(c_str(), length, s, & ss);
	*ss = 0;
	String result = String(s) + fileName;
	delete [] s;
	return result;
}

String String::replaceChar(wchar_t c, const String & s) const
{
	String result;
	for (const_iterator si = begin(); si != end(); si++) {
		if ((*si) == c) {
			result += s;
		} else {
			result += (*si);
		}
	}
	return result;
}

StringVector String::tokenize()
{
	StringVector v;
	String::iterator c = begin(); 
	while (c != end()) {
		if (iswalpha(*c)) {
			String s = (*c);
			while (++c != end() && iswalnum(*c)) s += (*c);
			v.push_back(s);
		} else if (iswdigit(*c)) {
			String s = (*c);
			while (++c != end() && (iswdigit(*c) || (*c) == L'.')) s += (*c);
			v.push_back(s);
		} else if (! iswspace(*c)) {
			v.push_back(String(*c++));
		} else {
			c++;
		}
	}
	return v;
}
