#ifndef STRING_H_
#define STRING_H_

#include <windows.h>

#include <string>
#include <vector>

#include "winutils/winutils.h"

class String : public std::wstring
{
public:
	String() : std::wstring() {};
	String(wchar_t * s, bool fatalFree = false) : std::wstring(s) {	if (fatalFree) FatalFree(s); };
	String(const std::wstring & s) : std::wstring(s) {};
	String(const String & s) : std::wstring(s) {};
	String(wchar_t c) : std::wstring() { *this = c; };
	String(int x) : std::wstring() { *this = x; };
	String(double x) : std::wstring() { *this = x; };
	virtual ~String() {};

public:
	String & operator=(wchar_t c) {
		std::wstring::operator=(c);
		return *this;
	};
	String & operator=(wchar_t * s) {
		std::wstring::operator=(s);
		return *this;
	};
	String & operator=(const std::wstring & s) {
		std::wstring::operator=(s);
		return *this;
	};
	String & operator=(const String & s) {
		std::wstring::operator=(s);
		return *this;
	};
	String & operator=(int x);
	String & operator=(double x);

	String & operator+=(wchar_t c) {
		std::wstring::operator+=(c);
		return *this;
	};
	String & operator+=(wchar_t * s) {
		std::wstring::operator+=(s);
		return *this;
	};
	String & operator+=(const std::wstring & s) {
		std::wstring::operator+=(s);
		return *this;
	};
	String & operator+=(const String & s) {
		std::wstring::operator+=(s);
		return *this;
	};
	String & operator+=(int x);
	String & operator+=(double x);

	const wchar_t & operator[](size_t pos) const {
		return std::wstring::operator[](pos);
	};
	wchar_t & operator[](size_t pos) {
		return std::wstring::operator[](pos);
	};

	bool operator == (const String & s) const {
		return (lstrcmpW(c_str(), s.c_str()) == 0);
	};
	bool operator != (const String & s) const {
		return (lstrcmpW(c_str(), s.c_str()) != 0);
	};
	bool operator <= (const String & s) const {
		return (lstrcmpW(c_str(), s.c_str()) <= 0);
	};
	bool operator >= (const String & s) const {
		return (lstrcmpW(c_str(), s.c_str()) >= 0);
	};
	bool operator < (const String & s) const {
		return (lstrcmpW(c_str(), s.c_str()) < 0);
	};
	bool operator > (const String & s) const {
		return (lstrcmpW(c_str(), s.c_str()) > 0);
	};

	String & fromWindow(HWND hWnd);
	String & fromLength(double length);
	String & fromUtf8(const std::string & s);
	String & fromUtf8(const char * s);
	String & fromAnsi(const std::string & s);
	String & fromAnsi(const char * s);
	std::string toUtf8() const;
	std::string toAnsi() const;

	String crlf() const;
	String lfOnly() const;
	String trim() const;
	String firstLine() const;
	String enclose(wchar_t c = L'"') const;
	String xmlenc() const;

	String toShortPath() const;
	String toFullPath() const;
	String toFileName() const;
	String replaceFileName(const String & fileName) const;
	String replaceChar(wchar_t c, const String & s) const;
	std::vector<String> tokenize();
};

typedef std::vector<String> StringVector;
typedef std::vector<StringVector> StringVectorVector;

#define S(s) String(TEXT(s))

#endif // STRING_H_
