#include "title.h"
#include "group.h"
#include "video.h"
#include "audio.h"

Title::Title(Group * group) : ManagablePtr()
{
	m_Group = group;
	m_ConvertToMono = NULL;
	m_Index = -1;
}

Title::~Title()
{
	delete [] m_ConvertToMono;
}

Group * Title::GetGroup() const
{
	return m_Group;
}

bool Title::IsSelected() const
{
	return GetVideo(0)->IsTitleSelected();
}

void Title::SetSelected(bool selected)
{
	for (int vindex = 0; vindex < GetVideoCount(); vindex++) {
		GetVideo(vindex)->SetTitleSelected(selected);
	}
}

void Title::AddVideo(VideoPtr video)
{
	m_Videos.push_back(video);
}

int Title::GetVideoCount() const
{
	return (int) m_Videos.size();
}

bool Title::IsVideoIndexValid(int index) const
{
	return (0 <= index && index < (int) m_Videos.size());
}

VideoPtr Title::GetVideo(int index) const
{
	if (IsVideoIndexValid(index)) {
		return m_Videos[index];
	} else {
		return VideoPtr();
	}
}

int Title::GetIndex() const
{
	return m_Index;
}

void Title::SetIndex(int index)
{
	m_Index = index;
}

String Title::GetIdent()
{
	return S("Title ") + String(GetIndex() + 1) 
		+ S(" of group ") + String(GetGroup()->GetIndex() + 1) + S(": ");
}

String Title::Recalculate()
{
	String error;
	bool first = true;
	bool interlaced = false;
	bool bff = false;
	int * noOfChannels = new int[m_Group->GetAudioCount()];
	delete [] m_ConvertToMono;
	m_ConvertToMono = new bool[m_Group->GetAudioCount()];
	m_Length = 0.0;
	for (int vindex = 0; vindex < GetVideoCount(); vindex++) {
		Video * video = GetVideo(vindex);
		video->SetIndex(vindex);
		error += video->Recalculate();
		if (first) {
			interlaced = video->IsInterlaced();
			bff = video->IsBFF();
			m_Deinterlace = false;
			for (int aindex = 0; aindex < m_Group->GetAudioCount(); aindex++) {
				noOfChannels[aindex] = video->GetAudio(aindex)->GetNoOfChannels();
				m_ConvertToMono[aindex] = false;
			}
			first = false;
		} else {
			if ((interlaced != video->IsInterlaced()) || (bff != video->IsBFF())) {
				m_Deinterlace = true;
			}
			for (int aindex = 0; aindex < m_Group->GetAudioCount(); aindex++) {
				if (noOfChannels[aindex] != video->GetAudio(aindex)->GetNoOfChannels()) {
					m_ConvertToMono[aindex] = true;
				}
			}
		}
		m_Length += video->GetLength();
	}
	delete [] noOfChannels;
	return error;
}

bool Title::GetDeinterlace() const
{
	return m_Deinterlace;
}

bool Title::GetConvertToMono(int index) const
{
	return m_ConvertToMono[index];
}

double Title::GetLength() const
{
	return m_Length;
}

void Title::GetDescription(StringVectorVector & strings) const
{
	StringVector line;
	line.push_back(S("Title ") + String(GetIndex() + 1) + S(": "));
	line.push_back(String().fromLength(GetLength()));
	strings.push_back(line);
}

