#ifndef TITLEDIALOG_H_
#define TITLEDIALOG_H_

#include "dialog.h"
#include "maindialog.h"
#include "group.h"

#define TITLE_OBJECT    0
#define VIDEO_OBJECT    1
#define AUDIO_OBJECT    2
#define SUBTITLE_OBJECT 3
#define VIDEO_TRACK     4
#define AUDIO_TRACK     5
#define SUBTITLE_TRACK  6

class TitleListItem : public ManagablePtr
{
public:
	TitleListItem(Group * group, int object, int index, int indent);
	TitleListItem(Title * title, int indent);
	TitleListItem(Video * video, bool isTrack, int indent);
	TitleListItem(Audio * audio, bool isTrack, int indent);
	TitleListItem(Subtitle * subtitle, bool isTrack, int indent);
	virtual ~TitleListItem();
	int GetDescription(StringVectorVector & strings) const;
	bool IsSelected(bool really = false) const;
	void SetSelected(bool selected);
	int GetObject() const;
	Group * GetGroup();
	Title * GetTitle();
	Video * GetVideo();
	Audio * GetAudio();
	void SetAudio(Audio * audio);
	Subtitle * GetSubtitle();
	void SetSubtitle(Subtitle * subtitle);
	bool IsTrack() const;
	int GetIndent() const;
	int GetNextIndent() const;
	void SetNextIndent(int nextIndent);
	void SetParent(TitleListItem * parent);

protected:
	int m_Object;
	int m_Index;
	GroupPtr m_Group;
	TitlePtr m_Title;
	VideoPtr m_Video;
	AudioPtr m_Audio;
	SubtitlePtr m_Subtitle;
	bool m_IsTrack;
	int m_Indent;
	int m_NextIndent;
	TitleListItem * m_Parent;
};
typedef ManagedPtr<TitleListItem> TitleListItemPtr;
typedef std::vector<TitleListItemPtr> TitleListItemVector;

class TitleDialog : public Dialog
{
public:
	TitleDialog(MainDialog * mainDialog, GroupPtr group, int groupIndex);
	virtual ~TitleDialog();

public:
	int DoModal();
	void Refresh();

protected:
	void ReconstructView();
	void ClearSelection();
	void UpdateVideoList(bool reset);
	void UpdateVideoTitle();
	void UpdateVideoPart();
	void UpdateAudioList();
	void UpdateAudioTitle();
	void UpdateAudioPart();
	void UpdateSubtitleList();
	void UpdateSubtitleTitle();
	void UpdateSubtitlePart();
	void Update(bool reset = false);
	bool DialogProc(UINT message, WPARAM wParam, LPARAM lParam);
	void InitDialog();
	void MeasureItem(LPMEASUREITEMSTRUCT mis);
	void DrawItem(LPDRAWITEMSTRUCT dis);
	void OnSelChange();
	void AddTitle();
	void BrowseAudioFile();
	void BrowseSubtitleFile();
	void TitleDialog::ChooseSubtitleColor();

protected:
	MainDialog * m_MainDialog;
	GroupPtr m_Group;
	int m_GroupIndex;
	TitleListItemVector m_TitleListItems;
};

#endif // TITLEDIALOG_H_
