#include "videoencoder.h"
#include "group.h"
#include "title.h"
#include "video.h"
#include "tools.h"
#include "config.h"
#include "main.h"
#include "config.h"
#include <stdio.h>
#include "winutils/winutils.h"

#include <map>

VideoEncoder::VideoEncoder(ProjectPtr project, int groupIndex, int titleIndex) : Thread()
{
	m_Project = project;
	m_GroupIndex = groupIndex;
	m_TitleIndex = titleIndex;
}

VideoEncoder::~VideoEncoder()
{
}

String VideoEncoder::GetDescription() const
{
	return S("Encoding video of title ") + String(m_TitleIndex + 1) + S(" in group ") + String(m_GroupIndex + 1);
}

bool VideoEncoder::OnRun()
{
	bool ok = true;
	
	GroupPtr group = m_Project->GetGroup(m_GroupIndex);
	TitlePtr title = group->GetTitle(m_TitleIndex);

	String baseFileName = S("set") + String(m_GroupIndex + 1) + 
		S("title") + String(m_TitleIndex + 1) + S("video");
	String avsFileName = String(m_Project->GetOutputDirectory() + S("\\") + baseFileName + S(".avs")).toFullPath();
	String m2vFileName = String(m_Project->GetOutputDirectory() + S("\\") + baseFileName + S(".m2v")).toFullPath();
	String logFileName = String(m_Project->GetOutputDirectory() + S("\\") + S("log.txt")).toFullPath();
	AddFileToDelete(avsFileName);

	bool tff = false;
	bool interlaced = false;

	FILE * f = fopen(avsFileName.toAnsi().c_str(), "wt");
	if (f) {
		fprintf(f, "LoadPlugin(%s)\n", tools.decomb.enclose().toAnsi().c_str());
		std::map<String, bool> imports;
		for (int vindex = 0; vindex < title->GetVideoCount(); vindex++) {
			VideoPtr video = title->GetVideo(vindex);
			if (video->GetOpener().length()) {
				Config::FunctionMap::const_iterator opener = g_Config->GetOpeners().find(video->GetOpener());
				if (opener != g_Config->GetOpeners().end()) imports[(*opener).second] = true;
			}
			if (video->GetPreProcessor().length()) {
				Config::FunctionMap::const_iterator preProcessor = g_Config->GetPreProcessors().find(video->GetPreProcessor());
				if (preProcessor != g_Config->GetPreProcessors().end()) imports[(*preProcessor).second] = true;
			}
		}
		for (std::map<String, bool>::iterator import = imports.begin(); import != imports.end(); import++) {
			fprintf(f, "Import(%s)\n", (*import).first.enclose().toAnsi().c_str());
		}
		for (int vindex = 0; vindex < title->GetVideoCount(); vindex++) {
			VideoPtr video = title->GetVideo(vindex);
			int width, height;
			int xborder = 0;
			int yborder = 0;
			int xcrop = 0;
			int ycrop = 0;
			double overscan = 1.0 - (video->GetCompensateOverscan() ? 0.01 * video->GetOverscanPercent() : 0.0); 
			double ratio = video->GetFinalAspectRatio() / video->GetOrigAspectRatio();
			if (ratio < 1.0) {
				width = (int)(0.75 + 0.25 * m_Project->GetVideoWidth() * overscan) * 4;
				height = (int)(0.75 + 0.25 * m_Project->GetVideoHeight() * ratio * overscan) * 4;
			} else {
				width = (int)(0.75 + 0.25 * m_Project->GetVideoWidth() / ratio * overscan) * 4;
				height = (int)(0.75 + 0.25 * m_Project->GetVideoHeight() * overscan) * 4;
			}
			xborder = (m_Project->GetVideoWidth() - width) / 2;
			yborder = (m_Project->GetVideoHeight() - height) / 2;
			if (video->IsScript()) {
				fprintf(f, "v%d = Import(%s)\n", vindex, video->GetFileName().enclose().toAnsi().c_str());
			} else {
				fprintf(f, "v%d = %s(%s, %s)\n", vindex, video->GetOpener().toAnsi().c_str(),
					video->GetFileName().enclose().toAnsi().c_str(),
					(m_Project->GetDVDFormat() == DVDFORMAT_PAL ? "true" : "false"));
			}
			if (video->GetUpsideDown()) {
				fprintf(f, "v%d = FlipVertical(v%d)\n", vindex, vindex);
			}
			if (video->GetPreProcessor().length()) {
				fprintf(f, "v%d = %s(v%d, %s)\n", vindex, video->GetPreProcessor().toAnsi().c_str(), vindex,
					(m_Project->GetDVDFormat() == DVDFORMAT_PAL ? "true" : "false"));
			}
			fprintf(f, "v%d = KillAudio(v%d)\n", vindex, vindex);
			switch (video->GetInterlacing()) 
			{
			case INTERLACING_AUTO:
				fprintf(f, "v%d = ConvertToYV12(v%d)\n", vindex, vindex);
				fprintf(f, "v%d = FieldDeinterlace(v%d, full=false)\n", vindex, vindex);
				break;
			case INTERLACING_PROGRESSIVE:
				fprintf(f, "v%d = ConvertToYV12(v%d, interlaced=false)\n", vindex, vindex);
				break;
			case INTERLACING_TFF:
				tff = true;
			case INTERLACING_BFF:
				fprintf(f, "v%d = ConvertToYV12(v%d, interlaced=true)\n", vindex, vindex);
				fprintf(f, "v%d = Assume%s(v%d)\n", vindex, (tff ? "TFF" : "BFF"), vindex);
				if (video->GetDoIVTC()) {
					fprintf(f, "v%d = Decimate(Telecide(v%d), cycle=5)\n", vindex, vindex);
				} else if (title->GetDeinterlace()) {
					fprintf(f, "v%d = FieldDeinterlace(v%d, full=true)\n", vindex, vindex);
				} else {
					interlaced = true;
					fprintf(f, "v%d = SeparateFields(v%d)\n", vindex, vindex);
					fprintf(f, "ev = SelectEven(v%d)\n", vindex);
					fprintf(f, "ov = SelectOdd(v%d)\n",  vindex);
					if (video->GetInfo().width != width || video->GetInfo().height != height) {
						fprintf(f, "ev = Lanczos4Resize(ev, %d, %d)\n", width, height / 2);
						fprintf(f, "ov = Lanczos4Resize(ov, %d, %d)\n", width, height / 2);
					}
					if (video->DoConvertFPS()) {
						fprintf(f, "ev = ChangeFPS(ev, %f)\n", video->ConvertToFPS());
						fprintf(f, "ov = ChangeFPS(ov, %f)\n", video->ConvertToFPS());
					}
					if (video->AssumeFPS()) {
						fprintf(f, "ev = AssumeFPS(ev, \"%s\")\n", (m_Project->GetDVDFormat() == DVDFORMAT_PAL ? "pal_video" : "ntsc_video"));
						fprintf(f, "ov = AssumeFPS(ov, \"%s\")\n", (m_Project->GetDVDFormat() == DVDFORMAT_PAL ? "pal_video" : "ntsc_video"));
					} else {
						fprintf(f, "ev = ConvertFPS(ev, \"%s\")\n", (m_Project->GetDVDFormat() == DVDFORMAT_PAL ? "pal_video" : "ntsc_video"));
						fprintf(f, "ov = ConvertFPS(ov, \"%s\")\n", (m_Project->GetDVDFormat() == DVDFORMAT_PAL ? "pal_video" : "ntsc_video"));
					}
					fprintf(f, "v%d = Interleave(ev, ov)\n", vindex);
					fprintf(f, "v%d = Assume%s(v%d)\n", vindex, (tff ? "TFF" : "BFF"), vindex);
					fprintf(f, "v%d = Weave(v%d)\n", vindex, vindex);
				}
			} 
			if (! interlaced) {
				if (video->GetInfo().width != width || video->GetInfo().height != height) {
					fprintf(f, "v%d = Lanczos4Resize(v%d, %d, %d)\n", vindex, vindex, width, height);
				}
				if (video->DoConvertFPS()) {
					fprintf(f, "v%d = ChangeFPS(v%d, %f)\n", vindex, vindex, video->ConvertToFPS());
				}
				if (video->AssumeFPS()) {
					fprintf(f, "v%d = AssumeFPS(v%d, \"%s\")\n", vindex, vindex,
						(m_Project->GetDVDFormat() == DVDFORMAT_PAL ? "pal_video" : "ntsc_video"));
				} else {
					fprintf(f, "v%d = ConvertFPS(v%d, \"%s\")\n", vindex, vindex,
						(m_Project->GetDVDFormat() == DVDFORMAT_PAL ? "pal_video" : "ntsc_video"));
				}
			}
			if (xborder || yborder) {
				fprintf(f, "v%d = AddBorders(v%d, %d, %d, %d, %d)\n", vindex, vindex, xborder, yborder, xborder, yborder);
			}
		}
		if (title->GetVideoCount() == 1) {
			fprintf(f, "return v0\n");
		} else {
			fprintf(f, "return AlignedSplice(v0");
			for (int vindex = 1; vindex < title->GetVideoCount(); vindex++) {
				fprintf(f, ", v%d", vindex);
			}
			fprintf(f, ")\n");
		}
		fclose(f);
	}

	if (g_Config->GetEncoder() == ENCODER_HC) {

		String iniFileName = String(m_Project->GetOutputDirectory() + S("\\") + baseFileName + S(".ini")).toFullPath();
		AddFileToDelete(iniFileName);
		String dbsFileName = m_Project->GetOutputDirectory();
		AddFileToDelete(dbsFileName);
		f = fopen(iniFileName.toAnsi().c_str(), "wt");
		if (f) {
			String options = g_Config->GetHCOptions().trim();
			if (options.length()) {
				fprintf(f, "%s\n", options.toAnsi().c_str());
			}
			fprintf(f, "*INFILE  %s\n", avsFileName.toAnsi().c_str());
			fprintf(f, "*OUTFILE %s\n", m2vFileName.toAnsi().c_str());
			fprintf(f, "*DBPATH  %s\n", dbsFileName.toAnsi().c_str());
			fprintf(f, "*LOGFILE %s\n", logFileName.toAnsi().c_str());
			switch (g_Config->GetHCProfile()) {
			case HCPROFILE_FAST:
				fprintf(f, "*PROFILE FAST\n");
				break;
			case HCPROFILE_NORMAL:
				fprintf(f, "*PROFILE NORMAL\n");
				break;
			case HCPROFILE_BEST:
				fprintf(f, "*PROFILE BEST\n");
				break;
			}
			fprintf(f, "*ASPECT %s\n", group->GetVideo(0)->GetFinalAspectRatioStr().toAnsi().c_str());
			fprintf(f, "*BITRATE %d\n", m_Project->GetVideoBitRate());
			fprintf(f, "*MAXBITRATE %d\n", m_Project->GetMaxVideoBitRate());
			fprintf(f, "*WAIT 0\n");
			if (interlaced) {
				fprintf(f, "*INTERLACED\n");
				fprintf(f, (tff ? "*TFF\n" : "*BFF\n"));
			} else {
				fprintf(f, "*PROGRESSIVE\n");
			}
			fprintf(f, "*AVSRELOAD\n");
			if (g_Config->GetHideEncoder()) {
				fprintf(f, "*SILENT\n");
			}
			fclose(f);
		}
		String hcCommand = S("-ini ") + iniFileName.enclose();
		ok = ok && ExecuteCommand(tools.HCenc.c_str(), hcCommand.c_str(), 
			NULL, NULL, NULL, isCancelled, waitingForUser);

	} else { // ENCODER_QU

		String quCommand = S("-auto -close");
		if (g_Config->GetHideEncoder()) quCommand += S(" -silent");
		String params = g_Config->GetQuParams().trim();
		if (params.length()) quCommand += S(" ") + params;
		quCommand += (g_Config->GetQu2Pass() ? S(" -2") : S(" -1"));
		quCommand += (g_Config->GetQuHighQuality() ? S(" -hq") : S(" -nohq"));
		quCommand += (g_Config->GetQuTrellis() ? S(" -trell") : S(" -notrell"));
		quCommand += (g_Config->GetQuExtreme() ? S(" -extreme") : S(" -noextreme"));
		if (interlaced) {
			quCommand += S(" -interlaced") + (tff ? S(" -tff") : S(" -bff"));
		} else {
			quCommand += S(" -nointerlaced");
		}
		quCommand += S(" -i ") + avsFileName.enclose();
		quCommand += S(" -o ") + m2vFileName.enclose();
		quCommand += S(" -b ") + String(m_Project->GetVideoBitRate());
		quCommand += S(" -aspectratio ") + group->GetVideo(0)->GetFinalAspectRatioStr();
		quCommand += S(" -maxbitrate 9600 -mpeg2mux nomux");
		ExecuteCommand(tools.QuEnc.c_str(), quCommand.c_str(), 
			NULL, NULL, NULL, isCancelled, waitingForUser);

	}

	if (ok) DeleteFilesNow();
	return ok;
}
